@@ -19,10 +19,10 @@
  *
  * Examples:
  \verbatim
-  "User 1 with password prompt" <sip:user@domain.com>
-  "User 2 with stored password" <sip:user@domain.com>;auth_pass=pass
-  "User 2 with ICE" <sip:user@1.2.3.4;transport=tcp>;medianat=ice
-  "User 3 with IPv6" <sip:user@[2001:df8:0:16:216:6fff:fe91:614c]:5070>
+  "User 1 with password prompt" <sip:user@example.com>
+  "User 2 with stored password" <sip:user@example.com>;auth_pass=pass
+  "User 2 with ICE" <sip:user@192.0.2.4;transport=tcp>;medianat=ice
+  "User 3 with IPv6" <sip:user@[2001:db8:0:16:216:6fff:fe91:614c]:5070>
  \endverbatim
  */
 
@@ -49,43 +49,60 @@ static int account_write_template(const char *file)
 			 "#    ;transport={udp,tcp,tls}\n"
 			 "#\n"
 			 "#  addr-params:\n"
-			 "#    ;answermode={manual,early,auto}\n"
+			 "#    ;100rel={yes,no,required}\n"
+			 "#    ;answermode={manual,early,auto,"
+					   "early-audio,early-video}\n"
+			 "#    ;answerdelay=0\n"
 			 "#    ;audio_codecs=opus/48000/2,pcma,...\n"
 			 "#    ;audio_source=alsa,default\n"
 			 "#    ;audio_player=alsa,default\n"
+			 "#    ;sip_autoanswer={yes, no}\n"
+			 "#    ;sip_autoanswer_beep={off, on, local}\n"
+			 "#    ;dtmfmode={rtpevent, info, auto}\n"
 			 "#    ;auth_user=username\n"
 			 "#    ;auth_pass=password\n"
 			 "#    ;call_transfer=no\n"
+			 "#    ;cert=cert.pem\n"
 			 "#    ;mediaenc={srtp,srtp-mand,srtp-mandf"
 			 ",dtls_srtp,zrtp}\n"
 			 "#    ;medianat={stun,turn,ice}\n"
+			 "#    ;rtcp_mux={yes, no}\n"
 			 "#    ;mwi=no\n"
 			 "#    ;outbound=\"sip:primary.example.com"
 			 ";transport=tcp\"\n"
 			 "#    ;outbound2=sip:secondary.example.com\n"
 			 "#    ;ptime={10,20,30,40,...}\n"
 			 "#    ;regint=3600\n"
+			 "#    ;fbregint=120\n"
 			 "#    ;prio={0,1,2,3,...}\n"
+			 "#    ;rwait=90\n"
 			 "#    ;pubint=0 (publishing off)\n"
 			 "#    ;regq=0.5\n"
 			 "#    ;sipnat={outbound}\n"
 			 "#    ;stunuser=STUN/TURN/ICE-username\n"
 			 "#    ;stunpass=STUN/TURN/ICE-password\n"
 			 "#    ;stunserver=stun:[user:pass]@host[:port]\n"
-			 "#    ;video_codecs=h264,h263,...\n"
+			 "#    ;inreq_allowed={yes, no}  # default: yes\n"
+			 "#    ;video_codecs=h264,vp8,...\n"
 			 "#\n"
 			 "# Examples:\n"
 			 "#\n"
-			 "#  <sip:user@domain.com;transport=tcp>"
+			 "#  <sip:user@example.com>"
 		         ";auth_pass=secret\n"
-			 "#  <sip:user@1.2.3.4;transport=tcp>"
+			 "#  <sip:user@example.com;transport=tcp>"
+		         ";auth_pass=secret\n"
+			 "#  <sip:user@192.0.2.4;transport=tcp>"
 		         ";auth_pass=secret\n"
 			 "#  <sip:user@"
-			 "[2001:df8:0:16:216:6fff:fe91:614c]:5070"
+			 "[2001:db8:0:16:216:6fff:fe91:614c]:5070"
 			 ";transport=tcp>;auth_pass=secret\n"
 			 "#\n"
-		       "\n"
-		       "#<sip:user@domain>;auth_pass=PASSWORD\n");
+			 "#\n"
+			 "# A very basic example\n"
+			 "#<sip:user@iptel.org>;auth_pass=PASSWORD\n"
+			 "#\n"
+			 "# A registrar-less account\n"
+			 "#<sip:alice@office>;regint=0\n");
 	if (r < 0)
 		err = ENOMEM;
 
@@ -106,7 +123,7 @@ static int account_write_template(const char *file)
  */
 static int line_handler(const struct pl *addr, void *arg)
 {
-	char buf[512];
+	char buf[1024];
 	struct ua *ua;
 	struct account *acc;
 	int err;
@@ -180,7 +197,7 @@ static int account_read_file(void)
 	if (re_snprintf(file, sizeof(file), "%s/accounts", path) < 0)
 		return ENOMEM;
 
-	if (!conf_fileexist(file)) {
+	if (!fs_isfile(file)) {
 
 		(void)fs_mkdir(path, 0700);
 
