@@ -10,7 +10,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <alsa/asoundlib.h>
-#include <pthread.h>
+#include <re_atomic.h>
 #include <re.h>
 #include <rem.h>
 #include <baresip.h>
@@ -18,8 +18,8 @@
 
 
 struct auplay_st {
-	pthread_t thread;
-	volatile bool run;
+	thrd_t thread;
+	RE_ATOMIC bool run;
 	snd_pcm_t *write;
 	void *sampv;
 	size_t sampc;
@@ -35,11 +35,10 @@ static void auplay_destructor(void *arg)
 	struct auplay_st *st = arg;
 
 	/* Wait for termination of other thread */
-	if (st->run) {
+	if (re_atomic_rlx(&st->run)) {
 		debug("alsa: stopping playback thread (%s)\n", st->device);
-		st->run = false;
-		snd_pcm_drop(st->write);
-		(void)pthread_join(st->thread, NULL);
+		re_atomic_rlx_set(&st->run, false);
+		thrd_join(st->thread, NULL);
 	}
 
 	if (st->write)
@@ -50,7 +49,7 @@ static void auplay_destructor(void *arg)
 }
 
 
-static void *write_thread(void *arg)
+static int write_thread(void *arg)
 {
 	struct auplay_st *st = arg;
 	struct auframe af;
@@ -59,9 +58,10 @@ static void *write_thread(void *arg)
 
 	num_frames = st->prm.srate * st->prm.ptime / 1000;
 
-	auframe_init(&af, st->prm.fmt, st->sampv, st->sampc);
+	auframe_init(&af, st->prm.fmt, st->sampv, st->sampc, st->prm.srate,
+		     st->prm.ch);
 
-	while (st->run) {
+	while (re_atomic_rlx(&st->run)) {
 		const int samples = num_frames;
 		void *sampv;
 
@@ -81,7 +81,7 @@ static void *write_thread(void *arg)
 			}
 		}
 		else if (n < 0) {
-			if (st->run)
+			if (re_atomic_rlx(&st->run))
 				warning("alsa: write error: %s\n",
 					snd_strerror((int) n));
 		}
@@ -91,9 +91,9 @@ static void *write_thread(void *arg)
 		}
 	}
 
-	snd_pcm_drain(st->write);
+	snd_pcm_drop(st->write);
 
-	return NULL;
+	return 0;
 }
 
 
@@ -157,10 +157,10 @@ int alsa_play_alloc(struct auplay_st **stp, const struct auplay *ap,
 		goto out;
 	}
 
-	st->run = true;
-	err = pthread_create(&st->thread, NULL, write_thread, st);
+	re_atomic_rlx_set(&st->run, true);
+	err = thread_create_name(&st->thread, "alsa_play", write_thread, st);
 	if (err) {
-		st->run = false;
+		re_atomic_rlx_set(&st->run, false);
 		goto out;
 	}
 
