@@ -112,9 +112,9 @@ static void vidframe_set_pixbuf(struct vidframe *f, const CVImageBufferRef b)
 #endif
 		{{1280, 720}, &AVCaptureSessionPreset1280x720}
 	};
-	int i, best = -1;
+	int best = -1;
 
-	for (i=ARRAY_SIZE(mapv)-1; i>=0; i--) {
+	for (int i=RE_ARRAY_SIZE(mapv)-1; i>=0; i--) {
 
 		NSString *preset = *mapv[i].preset;
 
@@ -158,7 +158,12 @@ static void vidframe_set_pixbuf(struct vidframe *f, const CVImageBufferRef b)
 		AVCaptureDeviceDiscoverySession
 		discoverySessionWithDeviceTypes:
 			@[AVCaptureDeviceTypeBuiltInWideAngleCamera,
+#if (__IPHONE_OS_VERSION_MIN_REQUIRED >= 170000 || \
+     __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000)
+			  AVCaptureDeviceTypeExternal]
+#else
 			  AVCaptureDeviceTypeExternalUnknown]
+#endif
 		mediaType:AVMediaTypeVideo
 		position:pos
 	];
@@ -314,19 +319,20 @@ static void destructor(void *arg)
 
 
 static int alloc(struct vidsrc_st **stp, const struct vidsrc *vs,
-		 struct media_ctx **ctx, struct vidsrc_prm *prm,
+		 struct vidsrc_prm *prm,
 		 const struct vidsz *size, const char *fmt,
 		 const char *dev, vidsrc_frame_h *frameh,
+		 vidsrc_packet_h *packeth,
 		 vidsrc_error_h *errorh, void *arg)
 {
 	NSAutoreleasePool *pool;
 	struct vidsrc_st *st;
 	int err = 0;
 
-	(void)ctx;
 	(void)prm;
 	(void)fmt;
 	(void)dev;
+	(void)packeth;
 	(void)errorh;
 	(void)vs;
 
@@ -403,7 +409,12 @@ static int module_init(void)
 		AVCaptureDeviceDiscoverySession
 		discoverySessionWithDeviceTypes:
 			@[AVCaptureDeviceTypeBuiltInWideAngleCamera,
+#if (__IPHONE_OS_VERSION_MIN_REQUIRED >= 170000 || \
+     __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000)
+			  AVCaptureDeviceTypeExternal]
+#else
 			  AVCaptureDeviceTypeExternalUnknown]
+#endif
 		mediaType:AVMediaTypeVideo
 		position:AVCaptureDevicePositionUnspecified
 	];
