@@ -17,7 +17,7 @@
  *
  * Video codecs using libavcodec
  *
- * This module implements H.263, H.264 and H.265 video codecs
+ * This module implements H.264 and H.265 video codecs
  * using libavcodec from FFmpeg or libav projects.
  *
  *
@@ -39,24 +39,20 @@
  */
 
 
-AVCodec *avcodec_h264enc;             /* optional; specified H.264 encoder */
-AVCodec *avcodec_h264dec;             /* optional; specified H.264 decoder */
-AVCodec *avcodec_h265enc;
-AVCodec *avcodec_h265dec;
+const AVCodec *avcodec_h264enc;      /* optional; specified H.264 encoder */
+const AVCodec *avcodec_h264dec;      /* optional; specified H.264 decoder */
+const AVCodec *avcodec_h265enc;
+const AVCodec *avcodec_h265dec;
 
 
-#if LIBAVUTIL_VERSION_MAJOR >= 56
 AVBufferRef *avcodec_hw_device_ctx = NULL;
 enum AVPixelFormat avcodec_hw_pix_fmt;
 enum AVHWDeviceType avcodec_hw_type = AV_HWDEVICE_TYPE_NONE;
-#endif
 
 
 int avcodec_resolve_codecid(const char *s)
 {
-	if (0 == str_casecmp(s, "H263"))
-		return AV_CODEC_ID_H263;
-	else if (0 == str_casecmp(s, "H264"))
+	if (0 == str_casecmp(s, "H264"))
 		return AV_CODEC_ID_H264;
 #ifdef AV_CODEC_ID_H265
 	else if (0 == str_casecmp(s, "H265"))
@@ -67,19 +63,6 @@ int avcodec_resolve_codecid(const char *s)
 }
 
 
-static int h263_fmtp_enc(struct mbuf *mb, const struct sdp_format *fmt,
-			 bool offer, void *arg)
-{
-	(void)offer;
-	(void)arg;
-
-	if (!mb || !fmt)
-		return 0;
-
-	return mbuf_printf(mb, "a=fmtp:%s CIF=1;CIF4=1\r\n", fmt->id);
-}
-
-
 static struct vidcodec h264 = {
 	.name      = "H264",
 	.variant   = "packetization-mode=0",
@@ -89,6 +72,7 @@ static struct vidcodec h264 = {
 	.dech      = avcodec_decode_h264,
 	.fmtp_ench = avcodec_h264_fmtp_enc,
 	.fmtp_cmph = avcodec_h264_fmtp_cmp,
+	.packetizeh= avcodec_packetize,
 };
 
 static struct vidcodec h264_1 = {
@@ -100,16 +84,7 @@ static struct vidcodec h264_1 = {
 	.dech      = avcodec_decode_h264,
 	.fmtp_ench = avcodec_h264_fmtp_enc,
 	.fmtp_cmph = avcodec_h264_fmtp_cmp,
-};
-
-static struct vidcodec h263 = {
-	.pt        = "34",
-	.name      = "H263",
-	.encupdh   = avcodec_encode_update,
-	.ench      = avcodec_encode,
-	.decupdh   = avcodec_decode_update,
-	.dech      = avcodec_decode_h263,
-	.fmtp_ench = h263_fmtp_enc,
+	.packetizeh= avcodec_packetize,
 };
 
 static struct vidcodec h265 = {
@@ -119,6 +94,7 @@ static struct vidcodec h265 = {
 	.ench      = avcodec_encode,
 	.decupdh   = avcodec_decode_update,
 	.dech      = avcodec_decode_h265,
+	.packetizeh= avcodec_packetize,
 };
 
 
@@ -129,17 +105,7 @@ static int module_init(void)
 	char h264dec[64] = "h264";
 	char h265enc[64] = "libx265";
 	char h265dec[64] = "hevc";
-#if LIBAVUTIL_VERSION_MAJOR >= 56
 	char hwaccel[64];
-#endif
-
-#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(53, 10, 0)
-	avcodec_init();
-#endif
-
-#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(58, 9, 100)
-	avcodec_register_all();
-#endif
 
 	conf_get_str(conf_cur(), "avcodec_h264enc", h264enc, sizeof(h264enc));
 	conf_get_str(conf_cur(), "avcodec_h264dec", h264dec, sizeof(h264dec));
@@ -164,9 +130,6 @@ static int module_init(void)
 		vidcodec_register(vidcodecl, &h264_1);
 	}
 
-	if (avcodec_find_decoder(AV_CODEC_ID_H263))
-		vidcodec_register(vidcodecl, &h263);
-
 	if (avcodec_h265enc || avcodec_h265dec)
 		vidcodec_register(vidcodecl, &h265);
 
@@ -188,14 +151,12 @@ static int module_init(void)
 		     avcodec_h265dec->name, avcodec_h265dec->long_name);
 	}
 
-#if LIBAVUTIL_VERSION_MAJOR >= 56
 	/* common for encode/decode */
 	if (0 == conf_get_str(conf_cur(), "avcodec_hwaccel",
 			      hwaccel, sizeof(hwaccel))) {
 
 		enum AVHWDeviceType type;
 		int ret;
-		int i;
 
 		info("avcodec: enable hwaccel using '%s'\n", hwaccel);
 
@@ -205,10 +166,17 @@ static int module_init(void)
 			warning("avcodec: Device type"
 				" '%s' is not supported.\n", hwaccel);
 
+			info("Available device types:\n");
+			while ((type = av_hwdevice_iterate_types(type))
+				!= AV_HWDEVICE_TYPE_NONE)
+				info("    %s\n",
+				     av_hwdevice_get_type_name(type));
+			info("\n");
+
 			return ENOSYS;
 		}
 
-		for (i = 0;; i++) {
+		for (int i = 0;; i++) {
 			const AVCodecHWConfig *config;
 
 			config = avcodec_get_hw_config(avcodec_h264dec, i);
@@ -243,7 +211,6 @@ static int module_init(void)
 
 		avcodec_hw_type = type;
 	}
-#endif
 
 	return 0;
 }
@@ -252,14 +219,11 @@ static int module_init(void)
 static int module_close(void)
 {
 	vidcodec_unregister(&h265);
-	vidcodec_unregister(&h263);
 	vidcodec_unregister(&h264);
 	vidcodec_unregister(&h264_1);
 
-#if LIBAVUTIL_VERSION_MAJOR >= 56
 	if (avcodec_hw_device_ctx)
 		av_buffer_unref(&avcodec_hw_device_ctx);
-#endif
 
 	return 0;
 }
