@@ -142,7 +142,7 @@ static int load_current_contact(struct contacts *contacts, const char *path)
 	if (re_snprintf(file, sizeof(file), "%s/current_contact", path) < 0)
 		return ENOMEM;
 
-	if (conf_fileexist(file)) {
+	if (fs_isfile(file)) {
 
 		f = fopen(file, "r");
 		if (!f)
@@ -273,7 +273,8 @@ static int write_template(const char *file)
 			 "\n"
 			 "# Access rules\n"
 			 "#\"Catch All\" <sip:*@*>;access=block\n"
-			 "\"Good Friend\" <sip:good@friend.com>;access=allow\n"
+			 "\"Good Friend\" <sip:good@example.com>;"
+				"access=allow\n"
 			 "\n"
 			 );
 
@@ -297,7 +298,7 @@ static int module_init(void)
 	if (re_snprintf(file, sizeof(file), "%s/contacts", path) < 0)
 		return ENOMEM;
 
-	if (!conf_fileexist(file)) {
+	if (!fs_isfile(file)) {
 
 		(void)fs_mkdir(path, 0700);
 
@@ -310,7 +311,7 @@ static int module_init(void)
 	if (err)
 		return err;
 
-	err = cmd_register(baresip_commands(), cmdv, ARRAY_SIZE(cmdv));
+	err = cmd_register(baresip_commands(), cmdv, RE_ARRAY_SIZE(cmdv));
 	if (err)
 		return err;
 
