@@ -10,6 +10,7 @@
 #include <openssl/crypto.h>
 #endif
 #include <re.h>
+#include <rem.h>
 #include <baresip.h>
 
 
@@ -56,12 +57,7 @@ static int print_system_info(struct re_printf *pf, void *arg)
 	err |= re_hprintf(pf, " Compiler: %s\n", __VERSION__);
 #endif
 
-#if defined (USE_OPENSSL) && (OPENSSL_VERSION_NUMBER < 0x10100000L)
-	err |= re_hprintf(pf, " OpenSSL:  %s\n",
-			  SSLeay_version(SSLEAY_VERSION));
-#endif
-
-#if defined (USE_OPENSSL) && (OPENSSL_VERSION_NUMBER >= 0x10100000L)
+#ifdef USE_OPENSSL
 	err |= re_hprintf(pf, " OpenSSL:  %s\n",
 			  OpenSSL_version(OPENSSL_VERSION));
 #endif
@@ -199,18 +195,18 @@ static void fileinfo_timeout(void *arg)
 
 	if (st->finished) {
 		info("debug_cmd: length = %1.3lf seconds\n", s);
-		ua_event(NULL, UA_EVENT_CUSTOM, NULL,
-			 "debug_cmd: length = %lf seconds", s);
+		module_event("debug_cmd", "aufileinfo", NULL, NULL,
+			 "length = %lf seconds", s);
 	}
 	else if (s > 0.) {
 		warning("debug_cmd: timeout, length > %1.3lf seconds\n", s);
-		ua_event(NULL, UA_EVENT_CUSTOM, NULL,
-			 "debug_cmd: timeout, length > %1.3lf seconds", s);
+		module_event("debug_cmd", "aufileinfo", NULL, NULL,
+			 "timeout length = %lf seconds", s);
 	}
 	else {
 		info("debug_cmd: timeout\n");
-		ua_event(NULL, UA_EVENT_CUSTOM, NULL,
-			 "debug_cmd: timeout");
+		module_event("debug_cmd", "aufileinfo", NULL, NULL,
+			 "timeout", s);
 	}
 
 	mem_deref(st);
@@ -259,9 +255,10 @@ static int cmd_aufileinfo(struct re_printf *pf, void *arg)
 	int err = 0;
 	char *path;
 	char aumod[16];
+	const char *file = carg->prm;
 	struct fileinfo_st *st = NULL;
 
-	if (!str_isset(carg->prm)) {
+	if (!str_isset(file)) {
 		re_hprintf(pf, "fileplay: filename not specified\n");
 		return EINVAL;
 	}
@@ -272,9 +269,17 @@ static int cmd_aufileinfo(struct re_printf *pf, void *arg)
 		return EINVAL;
 	}
 
-	re_sdprintf(&path, "%s/%s",
-			conf_config()->audio.audio_path,
-			carg->prm);
+	/* absolute path? */
+	if (file[0] == '/' ||
+	    !re_regex(file, strlen(file), "https://") ||
+	    !re_regex(file, strlen(file), "http://") ||
+	    !re_regex(file, strlen(file), "file://")) {
+		if (re_sdprintf(&path, "%s", file) < 0)
+			return ENOMEM;
+	}
+	else if (re_sdprintf(&path, "%s/%s",
+			conf_config()->audio.audio_path, file) < 0)
+		return ENOMEM;
 
 	/* prm->ptime == 0 means blocking mode for ausrc */
 	st = mem_zalloc(sizeof(*st), fileinfo_destruct);
@@ -284,9 +289,9 @@ static int cmd_aufileinfo(struct re_printf *pf, void *arg)
 	}
 
 	err = ausrc_alloc(&st->ausrc, baresip_ausrcl(),
-			NULL, aumod,
-			&st->prm, path,
-			fileinfo_read_handler, fileinfo_err_handler, st);
+			  aumod,
+			  &st->prm, path,
+			  fileinfo_read_handler, fileinfo_err_handler, st);
 	if (err) {
 		warning("debug_cmd: %s - ausrc %s does not support zero ptime "
 				"or reading source %s failed. (%m)\n",
@@ -294,7 +299,10 @@ static int cmd_aufileinfo(struct re_printf *pf, void *arg)
 		goto out;
 	}
 
-	tmr_start(&st->tmr, 5000, fileinfo_timeout, st);
+	if (st->finished)
+		fileinfo_timeout(st);
+	else
+		tmr_start(&st->tmr, 5000, fileinfo_timeout, st);
 out:
 	if (err)
 		mem_deref(st);
@@ -362,6 +370,8 @@ static int print_uuid(struct re_printf *pf, void *arg)
 
 
 static const struct cmd debugcmdv[] = {
+{"apistate",    0,       0, "User Agent state",       cmd_api_uastate     },
+{"aufileinfo",  0, CMD_PRM, "Audio file info",        cmd_aufileinfo      },
 {"conf_reload", 0,       0, "Reload config file",     reload_config       },
 {"config",      0,       0, "Print configuration",    cmd_config_print    },
 {"loglevel",   'v',      0, "Log level toggle",       cmd_log_level       },
@@ -370,13 +380,11 @@ static const struct cmd debugcmdv[] = {
 {"modules",     0,       0, "Module debug",           mod_debug           },
 {"netstat",    'n',      0, "Network debug",          cmd_net_debug       },
 {"play",        0, CMD_PRM, "Play audio file",        cmd_play_file       },
-{"aufileinfo",  0, CMD_PRM, "Audio file info",        cmd_aufileinfo      },
 {"sipstat",    'i',      0, "SIP debug",              cmd_sip_debug       },
 {"sysinfo",    's',      0, "System info",            print_system_info   },
 {"timers",      0,       0, "Timer debug",            tmr_status          },
 {"uastat",     'u',      0, "UA debug",               cmd_ua_debug        },
 {"uuid",        0,       0, "Print UUID",             print_uuid          },
-{"apistate",    0,       0, "User Agent state",       cmd_api_uastate     },
 };
 
 
@@ -388,7 +396,7 @@ static int module_init(void)
 	(void)time(&start_time);
 
 	err = cmd_register(baresip_commands(),
-			   debugcmdv, ARRAY_SIZE(debugcmdv));
+			   debugcmdv, RE_ARRAY_SIZE(debugcmdv));
 
 	return err;
 }
