@@ -23,9 +23,20 @@ static struct menu menu;
 
 enum {
 	MIN_RINGTIME = 1000,
+	TONE_DELAY   =   20,         /* Delay for starting tone in [ms]      */
 };
 
 
+struct filter_arg {
+	enum call_state state;
+	const struct call *exclude;
+	const struct call *match;
+	struct call *call;
+};
+
+
+static void menu_sel_other(struct call *exclude);
+
 static int menu_set_incall(bool incall)
 {
 	int err = 0;
@@ -49,12 +60,10 @@ static int menu_set_incall(bool incall)
 
 static void tmrstat_handler(void *arg)
 {
-	struct call *call;
 	(void)arg;
 
-	/* the UI will only show the current active call */
-	call = menu_callcur();
-	if (!call)
+	/* the UI will only show the current current call */
+	if (!menu.curcall)
 		return;
 
 	tmr_start(&menu.tmr_stat, 100, tmrstat_handler, 0);
@@ -63,7 +72,7 @@ static void tmrstat_handler(void *arg)
 		return;
 
 	if (STATMODE_OFF != menu.statmode) {
-		(void)re_fprintf(stderr, "%H\r", call_status, call);
+		(void)re_fprintf(stderr, "%H\r", call_status, menu.curcall);
 	}
 }
 
@@ -71,7 +80,7 @@ static void tmrstat_handler(void *arg)
 void menu_update_callstatus(bool incall)
 {
 	/* if there are any active calls, enable the call status view */
-	if (incall)
+	if (incall && menu_callcur())
 		tmr_start(&menu.tmr_stat, 100, tmrstat_handler, 0);
 	else
 		tmr_cancel(&menu.tmr_stat);
@@ -90,7 +99,8 @@ static char *errorcode_fb_aufile(uint16_t scode)
 	switch (scode) {
 
 	case 404: return "notfound.wav";
-	case 486: return "busy.wav";
+	case 486:
+	case 603: return "busy.wav";
 	case 487: return NULL; /* ignore */
 	default:  return "error.wav";
 	}
@@ -102,26 +112,109 @@ static char *errorcode_key_aufile(uint16_t scode)
 	switch (scode) {
 
 	case 404: return "notfound_aufile";
-	case 486: return "busy_aufile";
+	case 486:
+	case 603:
+		  return "busy_aufile";
 	case 487: return NULL; /* ignore */
 	default:  return "error_aufile";
 	}
 }
 
 
-static bool active_call_test(const struct call* call)
+static void limit_earlymedia(struct call* call, void *arg)
 {
+	enum sdp_dir ldir, ndir;
+	uint32_t maxcnt = 32;
+	bool update = false;
+	(void)arg;
+
+	if (!call_is_outgoing(call))
+		return;
+
+	ldir = sdp_media_ldir(stream_sdpmedia(audio_strm(call_audio(call))));
+	ndir = ldir;
+	conf_get_u32(conf_cur(), "menu_max_earlyaudio", &maxcnt);
+
+	if (menu.outcnt > maxcnt)
+		ndir = SDP_INACTIVE;
+	else if (menu.outcnt > 1)
+		ndir &= SDP_SENDONLY;
+
+	if (ndir != ldir) {
+		call_set_audio_ldir(call, ndir);
+		update = true;
+	}
+
+	/* video */
+	if (!call_video(call))
+		return;
+
+	ldir = sdp_media_ldir(stream_sdpmedia(video_strm(call_video(call))));
+	ndir = ldir;
+
+	maxcnt = 32;
+	conf_get_u32(conf_cur(), "menu_max_earlyvideo_rx", &maxcnt);
+	if (menu.outcnt > maxcnt)
+		ndir &= SDP_SENDONLY;
+
+	maxcnt = 32;
+	conf_get_u32(conf_cur(), "menu_max_earlyvideo_tx", &maxcnt);
+	if (menu.outcnt > maxcnt)
+		ndir &= SDP_RECVONLY;
+
+	if (ndir != ldir) {
+		call_set_video_ldir(call, ndir);
+		update = true;
+	}
+
+	if (update)
+		call_update_media(call);
+}
+
+
+static bool active_call_test(const struct call* call, void *arg)
+{
+	struct filter_arg *fa = arg;
+
+	if (call == fa->exclude)
+		return false;
+
 	return call_state(call) == CALL_STATE_ESTABLISHED &&
 			!call_is_onhold(call);
 }
 
 
+static bool established_call_test(const struct call* call, void *arg)
+{
+	struct filter_arg *fa = arg;
+
+	if (call == fa->exclude)
+		return false;
+
+	return call_state(call) == CALL_STATE_ESTABLISHED;
+}
+
+
+static bool outgoing_call_test(const struct call* call, void *arg)
+{
+	struct filter_arg *fa = arg;
+	enum call_state st;
+
+	if (call == fa->exclude)
+		return false;
+
+	st = call_state(call);
+	return  st == CALL_STATE_OUTGOING || st == CALL_STATE_RINGING ||
+		st == CALL_STATE_EARLY;
+}
+
+
 static void find_first_call(struct call *call, void *arg)
 {
-	struct call **callp = arg;
+	struct filter_arg *fa = arg;
 
-	if (!*callp)
-		*callp = call;
+	if (!fa->call)
+		fa->call = call;
 }
 
 
@@ -140,21 +233,22 @@ struct call *menu_find_call_state(enum call_state st)
 	return NULL;
 }
 
+
 /**
  * Search all User-Agents for a call that matches
  *
  * @param matchh  Optional match handler. If NULL, the last call of the first
  *                  User-Agent is returned
+ * @param exclude Call to exclude
  *
  * @return  A call that matches
  */
-struct call *menu_find_call(call_match_h *matchh)
+struct call *menu_find_call(call_match_h *matchh, const struct call *exclude)
 {
-	struct call *call = NULL;
+	struct filter_arg fa = {CALL_STATE_UNKNOWN, exclude, NULL, NULL};
 
-	uag_filter_calls(find_first_call, matchh, &call);
-
-	return call;
+	uag_filter_calls(find_first_call, matchh, &fa);
+	return fa.call;
 }
 
 
@@ -162,81 +256,161 @@ static void menu_stop_play(void)
 {
 	menu.play = mem_deref(menu.play);
 	menu.ringback = false;
+	tmr_cancel(&menu.tmr_play);
 }
 
 
-static void menu_play(const char *ckey, const char *fname, int repeat)
+static int menu_ovkey(char **key, const struct call *call, struct pl *suffix)
+{
+	int err;
+	struct mbuf *mb = mbuf_alloc(32);
+	if (!mb)
+		return ENOMEM;
+
+	err = mbuf_printf(mb, "%s-%r", call_id(call), suffix);
+	if (err)
+		goto out;
+
+	mb->pos = 0;
+	err = mbuf_strdup(mb, key, mb->end);
+out:
+	mem_deref(mb);
+	return err;
+}
+
+
+static int menu_ovkey_str(char **key, const struct call *call,
+			  const char *suffix)
+{
+	struct pl pl;
+	pl_set_str(&pl, suffix);
+	return menu_ovkey(key, call, &pl);
+}
+
+enum Device {
+	DEVICE_ALERT,
+	DEVICE_PLAYER
+};
+
+static bool menu_play(const struct call *call,
+		      const char *ckey,
+		      const char *fname,
+		      int repeat,
+		      enum Device device)
 {
 	struct config *cfg = conf_config();
 	struct player *player = baresip_player();
-
+	char *ovkey;
+	const char *ovaukey = NULL;
 	struct pl pl = PL_INIT;
 	char *file = NULL;
+	int err;
+	const char *play_mod = cfg->audio.alert_mod;
+	const char *play_dev = cfg->audio.alert_dev;
 
-	if (conf_get(conf_cur(), ckey, &pl))
-		pl_set_str(&pl, fname);
+	if (device == DEVICE_PLAYER) {
+		play_mod = cfg->audio.play_mod;
+		play_dev = cfg->audio.play_dev;
+	}
+
+	if (ckey) {
+		err = menu_ovkey_str(&ovkey, call, ckey);
+		if (!err) {
+			ovaukey = odict_string(menu.ovaufile, ovkey);
+			mem_deref(ovkey);
+		}
+
+		if (ovaukey && !strcmp(ovaukey, "none"))
+			return false;
+
+		if (ovaukey)
+			(void)conf_get(conf_cur(), ovaukey, &pl);
+
+		if (!pl_isset(&pl))
+			(void)conf_get(conf_cur(), ckey, &pl);
+	}
 
 	if (!pl_isset(&pl))
-		return;
+		pl_set_str(&pl, fname);
+
+	if (!pl_isset(&pl) || !pl_strcmp(&pl, "none"))
+		return false;
 
 	pl_strdup(&file, &pl);
 	menu_stop_play();
-	(void)play_file(&menu.play, player, file, repeat,
-			cfg->audio.play_mod,
-			cfg->audio.play_dev);
+	err = play_file(&menu.play, player, file, repeat,
+			play_mod,
+			play_dev);
 	mem_deref(file);
+
+	return err == 0;
 }
 
 
 static void play_incoming(const struct call *call)
 {
-	/* stop any ringtones */
-	menu_stop_play();
 
-	/* Only play the ringtones if answermode is "Manual".
-	 * If the answermode is "auto" then be silent.
-	 */
-	if (ANSWERMODE_MANUAL == account_answermode(call_account(call))) {
+	if (call_state(call) != CALL_STATE_INCOMING)
+		return;
 
-		if (menu_find_call(active_call_test)) {
-			menu_play("callwaiting_aufile", "callwaiting.wav", 3);
-		}
-		else {
-			/* Alert user */
-			menu_play("ring_aufile", "ring.wav", -1);
-		}
+	if (menu_find_call(active_call_test, call)) {
+		menu_play(call, "callwaiting_aufile", "callwaiting.wav", 3,
+			  DEVICE_PLAYER);
+	}
+	else if (menu.curcall == call) {
+		/* Alert user */
+		menu_play(call, "ring_aufile", "ring.wav", -1, DEVICE_ALERT);
 	}
 }
 
 
-static void play_ringback(void)
+static void play_ringback(const struct call *call)
 {
 	/* stop any ringtones */
 	menu_stop_play();
 
 	if (menu.ringback_disabled) {
-		info("\nRingback disabled\n");
+		info("menu: ringback disabled\n");
 	}
 	else {
-		menu_play("ringback_aufile", "ringback.wav", -1);
+		menu_play(call, "ringback_aufile", "ringback.wav", -1,
+			  DEVICE_PLAYER);
 		menu.ringback = true;
 	}
 }
 
 
-static void play_resume(void)
+static void check_ringback(struct call *call)
 {
-	struct call *call = uag_call_find(menu.callid);
+	enum sdp_dir adir = sdp_media_dir(stream_sdpmedia(
+					  audio_strm(call_audio(call))));
+	bool ring = !(adir & SDP_RECVONLY);
+
+	if (ring && !menu.ringback &&
+	    !menu_find_call(active_call_test, NULL)) {
+		play_ringback(call);
+	}
+	else if (!ring) {
+		menu_stop_play();
+	}
+}
+
+
+static void delayed_play(void *arg)
+{
+	struct call *call = menu_callcur();
+	(void) arg;
 
 	switch (call_state(call)) {
 	case CALL_STATE_INCOMING:
 		play_incoming(call);
 		break;
 	case CALL_STATE_RINGING:
-		if (!menu.ringback && !menu_find_call(active_call_test))
-			play_ringback();
+	case CALL_STATE_EARLY:
+		check_ringback(call);
 		break;
 	default:
+		menu_stop_play();
 		break;
 	}
 }
@@ -307,6 +481,56 @@ static void redial_handler(void *arg)
 }
 
 
+static void invite_handler(void *arg)
+{
+	(void)arg;
+
+	const char *uri = menu.invite_uri;
+
+	if (!str_isset(uri))
+		return;
+
+	int err;
+	err = ua_connect(uag_find_requri(uri), NULL, NULL, uri, VIDMODE_ON);
+	if (err)
+		warning("menu: call to %s failed (%m)\n", menu.invite_uri,
+			err);
+
+	menu.invite_uri = mem_deref(menu.invite_uri);
+}
+
+
+static void menu_invite(const char *prm)
+{
+	menu.invite_uri = mem_deref(menu.invite_uri);
+	int err = str_dup(&menu.invite_uri, prm);
+	if (err) {
+		warning("menu: call to %s failed (%m)\n", prm, err);
+		return;
+	}
+
+	tmr_start(&menu.tmr_invite, 0, invite_handler, NULL);
+}
+
+
+static int menu_autoanwser_call(struct call *call)
+{
+	struct call *outgoing;
+	if (menu_find_call(established_call_test, call))
+		return EINVAL;
+
+	outgoing = menu_find_call(outgoing_call_test, call);
+	if (outgoing) {
+		call_hangup(outgoing, 0, NULL);
+		ua_event(call_get_ua(outgoing), UA_EVENT_CALL_CLOSED, outgoing,
+			 "Outgoing call cancelled due to auto answer");
+		mem_deref(outgoing);
+	}
+
+	return call_answer(call, 200, VIDMODE_ON);
+}
+
+
 static void menu_play_closed(struct call *call)
 {
 	uint16_t scode;
@@ -321,7 +545,7 @@ static void menu_play_closed(struct call *call)
 		key = errorcode_key_aufile(scode);
 		fb = errorcode_fb_aufile(scode);
 
-		menu_play(key, fb, 1);
+		menu_play(call, key, fb, 1, DEVICE_ALERT);
 	}
 }
 
@@ -340,17 +564,43 @@ static void auans_play_finished(struct play *play, void *arg)
 }
 
 
-static void start_sip_autoanswer(struct call *call)
+static bool alert_uri_supported(const char *uri)
 {
+	if (!re_regex(uri, strlen(uri), "https://"))
+		return true;
+
+	if (!re_regex(uri, strlen(uri), "http://"))
+		return true;
+
+	if (!re_regex(uri, strlen(uri), "file://") && fs_isfile(uri + 7))
+		return true;
+
+	return false;
+}
+
+
+static void start_autoanswer(struct call *call)
+{
+	struct account *acc = call_account(call);
 	int32_t adelay = call_answer_delay(call);
-	bool beep = true;
+	const char *aluri = call_alerturi(call);
+	enum sipansbeep bmet = account_sipansbeep(acc);
+	bool beep = false;
 
 	if (adelay == -1)
 		return;
 
-	conf_get_bool(conf_cur(), "sip_autoanswer_beep", &beep);
+	if (bmet) {
+		if (bmet != SIPANSBEEP_LOCAL && aluri &&
+				alert_uri_supported(aluri))
+			beep = menu_play(call, NULL, aluri, 1, DEVICE_ALERT);
+
+		if (!beep)
+			beep = menu_play(call, "sip_autoanswer_aufile",
+					 "autoanswer.wav", 1, DEVICE_ALERT);
+	}
+
 	if (beep) {
-		menu_play("sip_autoanswer_aufile", "autoanswer.wav", 1);
 		play_set_finish_handler(menu.play, auans_play_finished, call);
 	}
 	else {
@@ -361,6 +611,47 @@ static void start_sip_autoanswer(struct call *call)
 }
 
 
+static bool ovaufile_del(struct le *le, void *arg)
+{
+	struct odict_entry *oe = le->data;
+	struct call *call = arg;
+	const char *id = call_id(call);
+
+	if (!strncmp(odict_entry_key(oe), id, str_len(id)))
+		mem_deref(oe);
+
+	return false;
+}
+
+
+static void process_module_event(struct call *call, const char *prm)
+{
+	int err;
+	struct pl module, event, data;
+	struct pl from, to;
+	char *ovkey;
+
+	err = re_regex(prm, strlen(prm), "[^,]*,[^,]*,[~]*",
+					 &module, &event, &data);
+	if (err)
+		return;
+
+	if (!pl_strcmp(&event, "override-aufile")) {
+		err = re_regex(data.p, data.l, "[^:]*:[~]*", &from, &to);
+		if (err)
+			return;
+
+		err = menu_ovkey(&ovkey, call, &from);
+		if (err)
+			return;
+
+		odict_entry_del(menu.ovaufile, ovkey);
+		odict_entry_add(menu.ovaufile, ovkey, ODICT_STRING, to.p);
+		mem_deref(ovkey);
+	}
+}
+
+
 static void ua_event_handler(struct ua *ua, enum ua_event ev,
 			     struct call *call, const char *prm, void *arg)
 {
@@ -370,8 +661,9 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 	bool incall;
 	enum sdp_dir ardir, vrdir;
 	uint32_t count;
+	struct pl val;
+	char * uri;
 	int err;
-	(void)prm;
 	(void)arg;
 
 #if 0
@@ -380,17 +672,28 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 #endif
 
 
+	ardir =sdp_media_rdir(
+			stream_sdpmedia(audio_strm(call_audio(call))));
 	count = uag_call_count();
+
 	switch (ev) {
 
 	case UA_EVENT_CALL_INCOMING:
 
-		/* set the current User-Agent to the one with the call */
-		menu_selcall(call);
-		menu_stop_play();
+		if (call_state(call) != CALL_STATE_INCOMING)
+			return;
+
+		if (account_answermode(acc) == ANSWERMODE_AUTO) {
+			if (!menu_autoanwser_call(call))
+				return;
+		}
+
+		/* the new incoming call should not change the current call */
+		if (!menu.curcall)
+			menu_selcall(call);
+		else
+			menu_selcall(menu.curcall);
 
-		ardir =sdp_media_rdir(
-			stream_sdpmedia(audio_strm(call_audio(call))));
 		vrdir = sdp_media_rdir(
 			stream_sdpmedia(video_strm(call_video(call))));
 		if (!call_has_video(call))
@@ -401,25 +704,41 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 		     account_aor(acc), call_peername(call), call_peeruri(call),
 		     sdp_dir_name(ardir), sdp_dir_name(vrdir));
 
-		if (acc && account_sip_autoanswer(acc))
+		if (account_sip_autoanswer(acc)) {
 			adelay = call_answer_delay(call);
+		}
+		else if (account_answerdelay(acc)) {
+			adelay = account_answerdelay(acc);
+			call_set_answer_delay(call, adelay);
+		}
 
 		if (adelay == -1)
 			play_incoming(call);
 		else
-			start_sip_autoanswer(call);
+			start_autoanswer(call);
 
 		break;
 
+	case UA_EVENT_CALL_OUTGOING:
+		++menu.outcnt;
+		break;
+
+	case UA_EVENT_CALL_LOCAL_SDP:
+		if (call_state(call) == CALL_STATE_OUTGOING)
+			menu_selcall(call);
+		break;
+
 	case UA_EVENT_CALL_RINGING:
 		menu_selcall(call);
-		if (!menu.ringback && !menu_find_call(active_call_test))
-			play_ringback();
+		if (!menu.ringback && !menu_find_call(active_call_test, call))
+			play_ringback(call);
 		break;
 
 	case UA_EVENT_CALL_PROGRESS:
 		menu_selcall(call);
-		menu_stop_play();
+		uag_filter_calls(limit_earlymedia, NULL, NULL);
+
+		tmr_start(&menu.tmr_play, TONE_DELAY, delayed_play, NULL);
 		break;
 
 	case UA_EVENT_CALL_ANSWERED:
@@ -469,18 +788,43 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 			}
 		}
 
-		if (!str_cmp(call_id(call), menu.callid)) {
-			menu_play_closed(call);
-			menu_selcall(NULL);
-			play_resume();
+		if (menu.xfer_call == call || menu.xfer_targ == call) {
+			call_hold(menu.xfer_call, false);
+			menu.xfer_call = NULL;
+			menu.xfer_targ = NULL;
 		}
 
+		if (call == menu.curcall) {
+			menu.curcall = NULL;
+			if (count == 1) {
+				menu_play_closed(call);
+			}
+			else {
+				menu_sel_other(call);
+				tmr_start(&menu.tmr_play, 0,
+					  delayed_play, NULL);
+			}
+		}
+		else if (call_state(call) == CALL_STATE_ESTABLISHED) {
+			tmr_start(&menu.tmr_play, 0,
+				  delayed_play, NULL);
+		}
+
+		hash_apply(menu.ovaufile->ht, ovaufile_del, call);
+		if (call_is_outgoing(call))
+			--menu.outcnt;
+
 		break;
 
 	case UA_EVENT_CALL_REMOTE_SDP:
 		if (!str_cmp(prm, "answer") &&
 				call_state(call) == CALL_STATE_ESTABLISHED)
 			menu_selcall(call);
+
+		if (call_state(call) == CALL_STATE_EARLY)
+			tmr_start(&menu.tmr_play, TONE_DELAY, delayed_play,
+				  NULL);
+
 		break;
 
 	case UA_EVENT_CALL_TRANSFER:
@@ -499,6 +843,7 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 		if (!err) {
 			struct pl pl;
 
+			call_set_user_data(call2, call_user_data(call));
 			pl_set_str(&pl, prm);
 
 			err = call_connect(call2, &pl);
@@ -506,6 +851,10 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 				warning("ua: transfer: connect error: %m\n",
 					err);
 			}
+			else {
+				module_event("menu", "transfer", ua, call,
+					     "target %s", call_id(call2));
+			}
 		}
 
 		if (err) {
@@ -516,6 +865,33 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 
 	case UA_EVENT_CALL_TRANSFER_FAILED:
 		info("menu: transfer failure: %s\n", prm);
+		menu_stop_play();
+		call_hold(call, false);
+		menu_selcall(call);
+		break;
+
+	case UA_EVENT_CALL_REDIRECT:
+		uri = strchr(prm, ',') + 1;
+		if (account_sip_autoredirect(ua_account(ua))) {
+			info("menu: redirecting call to %s\n", uri);
+			menu_invite(prm);
+		}
+		else {
+			info("menu: redirect call to %s\n", uri);
+		}
+		break;
+
+	case UA_EVENT_REFER:
+		val = pl_null;
+		if (!re_regex(prm, strlen(prm), "sip:"))
+			pl_set_str(&val, "invite");
+
+		(void)menu_param_decode(prm, "method", &val);
+		if (!pl_strcmp(&val, "invite")) {
+			info("menu: incoming REFER to %s\n", prm);
+			menu_invite(prm);
+		}
+
 		break;
 
 	case UA_EVENT_REGISTER_OK:
@@ -534,6 +910,10 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 		info("menu: audio error (%s)\n", prm);
 		break;
 
+	case UA_EVENT_MODULE:
+		process_module_event(call, prm);
+		break;
+
 	default:
 		break;
 	}
@@ -574,14 +954,43 @@ struct menu *menu_get(void)
 }
 
 
+static bool filter_call(const struct call *call, void *arg)
+{
+	struct filter_arg *fa = arg;
+
+	if (fa->state != CALL_STATE_UNKNOWN && call_state(call) != fa->state)
+		return false;
+
+	if (call == fa->exclude)
+		return false;
+
+	if (fa->match && call != fa->match)
+		return false;
+
+	return true;
+}
+
+
 /**
- * Selects the active call.
+ * Selects the given call to be the current call.
  *
  * @param call The call
  */
 void menu_selcall(struct call *call)
+{
+	menu.curcall = call;
+	call_set_current(ua_calls(call_get_ua(call)), call);
+}
+
+
+/**
+ * Chooses a new current call.
+ * Prefer call state established before early, ringing, outgoing and incoming
+ */
+static void menu_sel_other(struct call *exclude)
 {
 	int i;
+	struct filter_arg fa = {CALL_STATE_UNKNOWN, exclude, NULL, NULL};
 	enum call_state state[] = {
 		CALL_STATE_INCOMING,
 		CALL_STATE_OUTGOING,
@@ -590,25 +999,16 @@ void menu_selcall(struct call *call)
 		CALL_STATE_ESTABLISHED,
 	};
 
-	if (!call) {
-		/* select another call */
-		for (i = ARRAY_SIZE(state)-1; i >= 0; --i) {
-			call = menu_find_call_state(state[i]);
+	/* select another call */
+	for (i = RE_ARRAY_SIZE(state)-1; i >= 0; --i) {
+		fa.state = state[i];
+		uag_filter_calls(find_first_call, filter_call, &fa);
 
-			if (!str_cmp(call_id(call), menu.callid))
-				call = NULL;
-
-			if (call)
-				break;
-		}
+		if (fa.call)
+			break;
 	}
 
-	menu.callid = mem_deref(menu.callid);
-
-	if (call) {
-		str_dup(&menu.callid, call_id(call));
-		call_set_current(ua_calls(call_get_ua(call)), call);
-	}
+	menu_selcall(fa.call);
 }
 
 
@@ -619,7 +1019,13 @@ void menu_selcall(struct call *call)
  */
 struct call *menu_callcur(void)
 {
-	return uag_call_find(menu.callid);
+	struct filter_arg fa = {CALL_STATE_UNKNOWN, NULL, menu.curcall, NULL};
+
+	if (!menu.curcall)
+		return NULL;
+
+	uag_filter_calls(find_first_call, filter_call, &fa);
+	return fa.call;
 }
 
 
@@ -654,8 +1060,10 @@ struct ua   *menu_ua_carg(struct re_printf *pf, const struct cmd_arg *carg,
 	uint32_t i;
 	struct ua *ua = carg->data;
 
-	if (ua)
+	if (ua) {
+		pl_set_str(word1, carg->prm);
 		return ua;
+	}
 
 	err = re_regex(carg->prm, str_len(carg->prm), "[^ ]+ [^ ]+", word1,
 			word2);
@@ -724,6 +1132,9 @@ static int module_init(void)
 	menu.clean_number = false;
 	menu.play = NULL;
 	menu.adelay = -1;
+	err = odict_alloc(&menu.ovaufile, 8);
+	if (err)
+		return err;
 
 	/*
 	 * Read the config values
@@ -795,10 +1206,13 @@ static int module_close(void)
 
 	tmr_cancel(&menu.tmr_stat);
 	menu.dialbuf = mem_deref(menu.dialbuf);
-	menu.callid = mem_deref(menu.callid);
+	menu.invite_uri = mem_deref(menu.invite_uri);
+	menu.ovaufile = mem_deref(menu.ovaufile);
+	menu.ansval = mem_deref(menu.ansval);
 	menu_stop_play();
 
 	tmr_cancel(&menu.tmr_redial);
+	tmr_cancel(&menu.tmr_invite);
 
 	return 0;
 }
