@@ -6,33 +6,11 @@
 #include <stdlib.h>
 #include <re.h>
 #include <baresip.h>
+#include <string.h>
 
 #include "menu.h"
 
 
-/**
- * Decode a SDP direction
- *
- * @param pl  SDP direction as string
- *
- * @return sdp_dir SDP direction, SDP_SENDRECV as fallback
- */
-static enum sdp_dir decode_sdp_enum(const struct pl *pl)
-{
-	if (!pl_strcmp(pl, "inactive")) {
-		return SDP_INACTIVE;
-	}
-	else if (!pl_strcmp(pl, "sendonly")) {
-		return  SDP_SENDONLY;
-	}
-	else if (!pl_strcmp(pl, "recvonly")) {
-		return SDP_RECVONLY;
-	}
-
-	return SDP_SENDRECV;
-}
-
-
 static const char about_fmt[] =
 	".------------------------------------------------------------.\n"
 	"|                      "
@@ -93,7 +71,7 @@ static int cmd_answer(struct re_printf *pf, void *arg)
 	struct call *call = ua_call(ua);
 	int err;
 
-	if (carg->prm) {
+	if (str_isset(carg->prm)) {
 		call = uag_call_find(carg->prm);
 		if (!call) {
 			re_hprintf(pf, "call %s not found\n", carg->prm);
@@ -159,8 +137,8 @@ static int cmd_answerdir(struct re_printf *pf, void *arg)
 	if (!pl_isset(&argdir[1]))
 		argdir[1] = argdir[0];
 
-	adir = decode_sdp_enum(&argdir[0]);
-	vdir = decode_sdp_enum(&argdir[1]);
+	adir = sdp_dir_decode(&argdir[0]);
+	vdir = sdp_dir_decode(&argdir[1]);
 
 	if (adir == SDP_INACTIVE && vdir == SDP_INACTIVE) {
 		(void) re_hprintf(pf, "%s", usage);
@@ -180,7 +158,10 @@ static int cmd_answerdir(struct re_printf *pf, void *arg)
 		ua = call_get_ua(call);
 	}
 
-	(void)call_set_media_direction(call, adir, vdir);
+	call_set_media_estdir(call, adir, vdir);
+	if (call_sdp_change_allowed(call))
+		call_set_mdir(call, adir, vdir);
+
 	err = answer_call(ua, call);
 	if (err)
 		re_hprintf(pf, "could not answer call (%m)\n", err);
@@ -231,6 +212,86 @@ static int cmd_set_answermode(struct re_printf *pf, void *arg)
 }
 
 
+static int cmd_set_100rel_mode(struct re_printf *pf, void *arg)
+{
+	const struct cmd_arg *carg = arg;
+	struct pl w1 = PL_INIT;
+	struct pl w2 = PL_INIT;
+	struct ua *ua = menu_ua_carg(pf, carg, &w1, &w2);
+	char *mode_str = NULL;
+	enum rel100_mode mode;
+	struct le *le;
+	int err;
+
+	err = pl_strdup(&mode_str, &w1);
+	if (err) {
+		(void)re_hprintf(pf, "usage: /100rel <yes|no|required>"
+				 " [ua-idx]\n");
+		err = EINVAL;
+		goto out;
+	}
+
+	if (0 == str_cmp(mode_str, "no")) {
+		mode = REL100_DISABLED;
+	}
+	else if (0 == str_cmp(mode_str, "yes")) {
+		mode = REL100_ENABLED;
+	}
+	else if (0 == str_cmp(mode_str, "required")) {
+		mode = REL100_REQUIRED;
+	}
+	else {
+		(void)re_hprintf(pf, "Invalid 100rel mode: %s\n", mode_str);
+		err = EINVAL;
+		goto out;
+	}
+
+	if (!ua)
+		ua = uag_find_requri_pl(&w2);
+
+	if (ua) {
+		if (mode == account_rel100_mode(ua_account(ua)))
+			goto out;
+
+		err = account_set_rel100_mode(ua_account(ua), mode);
+		if (err)
+			goto out;
+
+		if (mode == REL100_DISABLED)
+			ua_remove_extension(ua, "100rel");
+		else
+			ua_add_extension(ua, "100rel");
+
+		(void)re_hprintf(pf, "100rel mode of account %s changed to: "
+				 "%s\n", account_aor(ua_account(ua)),
+				 mode_str);
+	}
+	else {
+		for (le = list_head(uag_list()); le; le = le->next) {
+			ua = le->data;
+
+			if (mode == account_rel100_mode(ua_account(ua)))
+				continue;
+
+			err = account_set_rel100_mode(ua_account(ua), mode);
+			if (err)
+				goto out;
+
+			if (mode == REL100_DISABLED)
+				ua_remove_extension(ua, "100rel");
+			else
+				ua_add_extension(ua, "100rel");
+		}
+		(void)re_hprintf(pf, "100rel mode of all accounts changed to: "
+				 "%s\n", mode_str);
+	}
+
+out:
+	mem_deref(mode_str);
+	return err;
+}
+
+
 static int switch_audio_player(struct re_printf *pf, void *arg)
 {
 	const struct cmd_arg *carg = arg;
@@ -411,50 +472,6 @@ static int ua_print_call_status(struct re_printf *pf, void *arg)
 }
 
 
-static void clean_number(char *str)
-{
-	int i = 0, k = 0;
-
-	/* only clean numeric numbers
-	 * In other cases trust the user input
-	 */
-	int err = re_regex(str, str_len(str), "[A-Za-z]");
-	if (err == 0)
-		return;
-
-	/* remove (0) which is in some mal-formated numbers
-	 * but only if trailed by another character
-	 */
-	if (str[0] == '+' || (str[0] == '0' && str[1] == '0'))
-		while (str[i]) {
-			if (str[i] == '('
-			 && str[i+1] == '0'
-			 && str[i+2] == ')'
-			 && (str[i+3] == ' '
-				 || (str[i+3] >= '0' && str[i+3] <= '9')
-			    )
-			) {
-				str[i+1] = ' ';
-				break;
-			}
-			++i;
-		}
-	i = 0;
-	while (str[i]) {
-		if (str[i] == ' '
-		 || str[i] == '.'
-		 || str[i] == '-'
-		 || str[i] == '/'
-		 || str[i] == '('
-		 || str[i] == ')')
-			++i;
-		else
-			str[k++] = str[i++];
-	}
-	str[k] = '\0';
-}
-
-
 static enum answer_method auto_answer_method(struct re_printf *pf)
 {
 	struct pl met;
@@ -488,7 +505,8 @@ static int dial_handler(struct re_printf *pf, void *arg)
 	struct pl word[2] = {PL_INIT, PL_INIT};
 	struct ua *ua = menu_ua_carg(pf, carg, &word[0], &word[1]);
 	char *uri = NULL;
-	struct mbuf *uribuf = NULL;
+	char *uric = NULL;
+	struct pl pluri;
 	struct call *call;
 	int err = 0;
 
@@ -518,9 +536,15 @@ static int dial_handler(struct re_printf *pf, void *arg)
 		if (menu->clean_number)
 			clean_number(uri);
 	}
+	else {
+		re_hprintf(pf, "can't find a URI to dial to\n");
+		err = EINVAL;
+		goto out;
+	}
 
+	pl_set_str(&pluri, uri);
 	if (!ua)
-		ua = uag_find_requri(uri);
+		ua = uag_find_requri_pl(&pluri);
 
 	if (!ua) {
 		re_hprintf(pf, "could not find UA for %s\n", uri);
@@ -528,29 +552,18 @@ static int dial_handler(struct re_printf *pf, void *arg)
 		goto out;
 	}
 
-	uribuf = mbuf_alloc(64);
-	if (!uribuf)
-		return ENOMEM;
-
-	err = account_uri_complete(ua_account(ua), uribuf, uri);
-	if (err) {
-		(void)re_hprintf(pf, "ua_connect failed to complete uri\n");
-		return EINVAL;
+	if (menu->adelay >= 0) {
+		ua_set_autoanswer_value(ua, menu->ansval);
+		(void)ua_enable_autoanswer(ua, menu->adelay,
+				auto_answer_method(pf));
 	}
 
-	mem_deref(uri);
-
-	uribuf->pos = 0;
-	err = mbuf_strdup(uribuf, &uri, uribuf->end);
+	re_hprintf(pf, "call uri: %s\n", uri);
+	err = account_uri_complete_strdup(ua_account(ua), &uric, &pluri);
 	if (err)
 		goto out;
 
-	if (menu->adelay >= 0)
-		(void)ua_enable_autoanswer(ua, menu->adelay,
-				auto_answer_method(pf));
-
-
-	err = ua_connect(ua, &call, NULL, uri, VIDMODE_ON);
+	err = ua_connect(ua, &call, NULL, uric, VIDMODE_ON);
 
 	if (menu->adelay >= 0)
 		(void)ua_disable_autoanswer(ua, auto_answer_method(pf));
@@ -559,11 +572,21 @@ static int dial_handler(struct re_printf *pf, void *arg)
 		goto out;
 	}
 
+	const char ud_sentinel[] = "userdata=";
+	char *ud_pos = NULL;
+	if (carg->prm != NULL)
+		ud_pos = strstr(carg->prm, ud_sentinel);
+	char *user_data = NULL;
+	if (ud_pos != NULL) {
+		user_data = ud_pos + strlen(ud_sentinel);
+		call_set_user_data(call, user_data);
+	}
+
 	re_hprintf(pf, "call id: %s\n", call_id(call));
 
 out:
-	mem_deref(uribuf);
 	mem_deref(uri);
+	mem_deref(uric);
 	return err;
 }
 
@@ -571,12 +594,13 @@ out:
 static int cmd_dialdir(struct re_printf *pf, void *arg)
 {
 	const struct cmd_arg *carg = arg;
+	struct menu *menu = menu_get();
 	enum sdp_dir adir, vdir;
 	struct pl argdir[2] = {PL_INIT, PL_INIT};
+	struct pl dname = PL_INIT;
 	struct pl pluri;
 	struct call *call;
 	char *uri = NULL;
-	struct mbuf *uribuf = NULL;
 	struct ua *ua = carg->data;
 	int err = 0;
 
@@ -588,14 +612,33 @@ static int cmd_dialdir(struct re_printf *pf, void *arg)
 			"Audio & video must not be"
 			" inactive at the same time\n";
 
+	/* full form with display name */
 	err = re_regex(carg->prm, str_len(carg->prm),
-		"[^ ]* audio=[^ ]* video=[^ ]*",
-		&pluri, &argdir[0], &argdir[1]);
-	if (err)
+		"[~ \t\r\n<]*[ \t\r\n]*<[^>]+>[ \t\r\n]*"
+		"audio=[^ \t\r\n]*[ \t\r\n]*video=[^ \t\r\n]*",
+		&dname, NULL, &pluri, NULL, &argdir[0], NULL, &argdir[1]);
+	if (err) {
+		dname = pl_null;
 		err = re_regex(carg->prm, str_len(carg->prm),
-			"[^ ]* [^ ]*",&pluri, &argdir[0]);
+			       "[^ ]+ audio=[^ ]* video=[^ ]*",
+			       &pluri, &argdir[0], &argdir[1]);
+	}
+
+	/* short form with display name */
+	if (err) {
+		err = re_regex(carg->prm, str_len(carg->prm),
+			       "[~ \t\r\n<]*[ \t\r\n]*<[^>]+>[ \t\r\n]+"
+			       "[^ \t\r\n]*",
+			       &dname, NULL, &pluri, NULL, &argdir[0]);
+	}
 
 	if (err) {
+		dname = pl_null;
+		err = re_regex(carg->prm, str_len(carg->prm),
+			       "[^ ]* [^ ]*",&pluri, &argdir[0]);
+	}
+
+	if (err || !re_regex(argdir[0].p, argdir[0].l, "=")) {
 		(void)re_hprintf(pf, "%s", usage);
 		return EINVAL;
 	}
@@ -603,20 +646,16 @@ static int cmd_dialdir(struct re_printf *pf, void *arg)
 	if (!pl_isset(&argdir[1]))
 		argdir[1] = argdir[0];
 
-	adir = decode_sdp_enum(&argdir[0]);
-	vdir = decode_sdp_enum(&argdir[1]);
+	adir = sdp_dir_decode(&argdir[0]);
+	vdir = sdp_dir_decode(&argdir[1]);
 
 	if (adir == SDP_INACTIVE && vdir == SDP_INACTIVE) {
 		(void)re_hprintf(pf, "%s", usage);
 		return EINVAL;
 	}
 
-	err = pl_strdup(&uri, &pluri);
-	if (err)
-		goto out;
-
 	if (!ua)
-		ua = uag_find_requri(carg->prm);
+		ua = uag_find_requri_pl(&pluri);
 
 	if (!ua) {
 		(void)re_hprintf(pf, "could not find UA for %s\n", carg->prm);
@@ -624,31 +663,45 @@ static int cmd_dialdir(struct re_printf *pf, void *arg)
 		goto out;
 	}
 
-	uribuf = mbuf_alloc(64);
-	if (!uribuf)
-		return ENOMEM;
-
-	err = account_uri_complete(ua_account(ua), uribuf, uri);
-	if (err) {
-		(void)re_hprintf(pf, "ua_connect failed to complete uri\n");
-		return EINVAL;
+	if (pl_isset(&dname)) {
+		err = re_sdprintf(&uri, "\"%r\" <%r>", &dname, &pluri);
+	}
+	else {
+		err = account_uri_complete_strdup(ua_account(ua), &uri,
+						  &pluri);
 	}
 
-	mem_deref(uri);
-
-	uribuf->pos = 0;
-	err = mbuf_strdup(uribuf, &uri, uribuf->end);
-	if (err)
+	if (err) {
+		(void)re_hprintf(pf, "ua_connect failed to complete uri\n");
 		goto out;
+	}
+
+	if (menu->adelay >= 0) {
+		ua_set_autoanswer_value(ua, menu->ansval);
+		(void)ua_enable_autoanswer(ua, menu->adelay,
+				auto_answer_method(pf));
+	}
+
+	re_hprintf(pf, "call uri: %s\n", uri);
 
 	err = ua_connect_dir(ua, &call, NULL, uri, VIDMODE_ON, adir, vdir);
+
+	if (menu->adelay >= 0)
+		(void)ua_disable_autoanswer(ua, auto_answer_method(pf));
 	if (err)
 		goto out;
 
+	const char ud_sentinel[] = "userdata=";
+	char *ud_pos = strstr(carg->prm, ud_sentinel);
+	char *user_data = NULL;
+	if (ud_pos != NULL) {
+		user_data = ud_pos + strlen(ud_sentinel);
+		call_set_user_data(call, user_data);
+	}
+
 	re_hprintf(pf, "call id: %s\n", call_id(call));
 
  out:
-	mem_deref(uribuf);
 	mem_deref(uri);
 
 	return err;
@@ -659,27 +712,63 @@ static int cmd_dnd(struct re_printf *pf, void *arg)
 {
 	int err = 0;
 	const struct cmd_arg *carg = arg;
+	bool en = false;
 
-	if (!str_isset(carg->prm)) {
-		err = EINVAL;
+	err = str_bool(&en, carg->prm);
+	if (err)
 		goto out;
-	}
 
-	if (!str_cmp(carg->prm, "true"))
-		uag_set_dnd(true);
-	else if (!str_cmp(carg->prm, "false"))
-		uag_set_dnd(false);
-	else
-		err = EINVAL;
+	uag_set_dnd(en);
 
  out:
 	if (err)
-		re_hprintf(pf, "usage: /dnd <true|false>\n");
+		re_hprintf(pf, "usage: /dnd <yes|no>\n");
 
 	return err;
 }
 
 
+static int cmd_enable_transp(struct re_printf *pf, void *arg)
+{
+	struct pl word1, word2;
+	int err = 0;
+	const struct cmd_arg *carg = arg;
+	bool en = true;
+	enum sip_transp tp;
+	char *buf = NULL;
+	const char *usage = "usage: /entransp <udp|tcp|tls|ws|wss> <yes|no>\n";
+
+	err = re_regex(carg->prm, str_len(carg->prm), "[^ ]+ [^ ]+", &word1,
+			&word2);
+	if (err) {
+		re_hprintf(pf, "%s", usage);
+		return EINVAL;
+	}
+
+	tp = sip_transp_decode(&word1);
+	if (tp == SIP_TRANSP_NONE) {
+		re_hprintf(pf, "%s", usage);
+		return EINVAL;
+	}
+
+	err = pl_strdup(&buf, &word2);
+	if (err)
+		return err;
+
+	err = str_bool(&en, buf);
+	if (err) {
+		re_hprintf(pf, "%s", usage);
+		goto out;
+	}
+
+	err = uag_enable_transport(tp, en);
+
+ out:
+	mem_deref(buf);
+	return err;
+}
+
+
 /**
  * Hangup the active call
  *
@@ -699,7 +788,7 @@ static int cmd_hangup(struct re_printf *pf, void *arg)
 
 	(void)pf;
 
-	if (carg->prm) {
+	if (str_isset(carg->prm)) {
 		call = uag_call_find(carg->prm);
 		if (!call) {
 			re_hprintf(pf, "call %s not found\n", carg->prm);
@@ -852,6 +941,20 @@ static void options_resp_handler(int err, const struct sip_msg *msg, void *arg)
 }
 
 
+static void refer_resp_handler(int err, const struct sip_msg *msg, void *arg)
+{
+	(void)arg;
+
+	if (err) {
+		warning("REFER reply error (%m)\n", err);
+		return;
+	}
+
+	info("%r: REFER reply %u %r\n", &msg->to.auri,
+	     msg->scode, &msg->reason);
+}
+
+
 static int options_command(struct re_printf *pf, void *arg)
 {
 	const struct cmd_arg *carg = arg;
@@ -861,34 +964,16 @@ static int options_command(struct re_printf *pf, void *arg)
 	struct mbuf *uribuf = NULL;
 	int err = 0;
 
-	err = pl_strdup(&uri, &word[0]);
-	if (err)
-		goto out;
-
 	if (!ua)
-		ua = uag_find_requri(uri);
+		ua = uag_find_requri_pl(&word[0]);
 
 	if (!ua) {
-		(void)re_hprintf(pf, "could not find UA for %s\n", uri);
+		(void)re_hprintf(pf, "could not find UA for %r\n", &word[0]);
 		err = EINVAL;
 		goto out;
 	}
 
-	uribuf = mbuf_alloc(64);
-	if (!uribuf)
-		return ENOMEM;
-
-	err = account_uri_complete(ua_account(ua), uribuf, uri);
-	if (err) {
-		(void)re_hprintf(pf, "options_command failed to "
-				 "complete uri\n");
-		return EINVAL;
-	}
-
-	mem_deref(uri);
-
-	uribuf->pos = 0;
-	err = mbuf_strdup(uribuf, &uri, uribuf->end);
+	err = account_uri_complete_strdup(ua_account(ua), &uri, &word[0]);
 	if (err)
 		goto out;
 
@@ -905,6 +990,53 @@ out:
 }
 
 
+static int cmd_refer(struct re_printf *pf, void *arg)
+{
+	const struct cmd_arg *carg = arg;
+	struct pl to;
+	struct pl referto;
+	struct ua *ua = carg->data;
+	char *uri = NULL;
+	char *touri = NULL;
+	struct mbuf *uribuf = NULL;
+	const char *usage = "usage: /refer <uri> <referto>\n";
+	int err = 0;
+
+	err = re_regex(carg->prm, str_len(carg->prm), "[^ ]+ [^ ]+",
+		       &to, &referto);
+	if (err) {
+		re_hprintf(pf, usage);
+		return err;
+	}
+
+	if (!ua)
+		ua = uag_find_requri_pl(&to);
+
+	if (!ua) {
+		(void)re_hprintf(pf, "could not find UA for %r\n", &to);
+		err = EINVAL;
+		goto out;
+	}
+
+	err  = account_uri_complete_strdup(ua_account(ua), &uri, &to);
+	err |= account_uri_complete_strdup(ua_account(ua), &touri, &referto);
+	if (err)
+		goto out;
+
+	err = ua_refer_send(ua, uri, touri, refer_resp_handler, NULL);
+
+out:
+	mem_deref(uribuf);
+	mem_deref(uri);
+	mem_deref(touri);
+	if (err) {
+		(void)re_hprintf(pf, "could not send REFER (%m)\n", err);
+	}
+
+	return err;
+}
+
+
 /**
  * Print the SIP Registration for all User-Agents
  *
@@ -952,7 +1084,7 @@ static int cmd_set_adelay(struct re_printf *pf, void *arg)
 {
 	const struct cmd_arg *carg = arg;
 
-	if (!carg->prm) {
+	if (!str_isset(carg->prm)) {
 		menu_get()->adelay = -1;
 		return 0;
 	}
@@ -968,6 +1100,35 @@ static int cmd_set_adelay(struct re_printf *pf, void *arg)
 }
 
 
+/**
+ * Set SIP auto answer Call-Info/Alert-Info value for outgoing calls
+ *
+ * @param pf     Print handler for debug output
+ * @param arg    Optional string that should be used as value for Call-Info/
+ *               Alert-Info header.
+ *               If no argument is specified, then the value is cleared.
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+static int cmd_set_ansval(struct re_printf *pf, void *arg)
+{
+	const struct cmd_arg *carg = arg;
+
+	menu_get()->ansval = mem_deref(menu_get()->ansval);
+	if (!str_isset(carg->prm))
+		return 0;
+
+	str_dup(&menu_get()->ansval, carg->prm);
+	if (menu_get()->ansval)
+		(void)re_hprintf(pf, "SIP auto answer value changed to %s\n",
+				 menu_get()->ansval);
+	else
+		(void)re_hprintf(pf, "SIP auto answer value cleared\n");
+
+	return 0;
+}
+
+
 static int cmd_ua_delete(struct re_printf *pf, void *arg)
 {
 	const struct cmd_arg *carg = arg;
@@ -995,8 +1156,7 @@ static int cmd_ua_delete_all(struct re_printf *pf, void *unused)
 
 	(void)unused;
 
-	while (list_head(uag_list()))
-	{
+	while (list_head(uag_list())) {
 		ua = list_head(uag_list())->data;
 		mem_deref(ua);
 	}
@@ -1010,6 +1170,7 @@ static int cmd_ua_delete_all(struct re_printf *pf, void *unused)
 static int cmd_ua_find(struct re_printf *pf, void *arg)
 {
 	const struct cmd_arg *carg = arg;
+	struct le *le;
 	struct ua *ua = NULL;
 
 	if (str_isset(carg->prm)) {
@@ -1024,6 +1185,12 @@ static int cmd_ua_find(struct re_printf *pf, void *arg)
 
 	(void)re_hprintf(pf, "ua: %s\n", account_aor(ua_account(ua)));
 
+	ua_raise(ua);
+
+	le = list_tail(ua_calls(ua));
+	if (le)
+		menu_selcall(le->data);
+
 	menu_update_callstatus(uag_call_count());
 
 	return 0;
@@ -1064,6 +1231,102 @@ static int create_ua(struct re_printf *pf, void *arg)
 }
 
 
+static int cmd_uareg(struct re_printf *pf, void *arg)
+{
+	const struct cmd_arg *carg = arg;
+	struct pl word[2] = {PL_INIT, PL_INIT};
+	struct ua *ua = menu_ua_carg(pf, carg, &word[0], &word[1]);
+	struct account *acc;
+	uint32_t regint;
+	int err;
+
+	if (!ua)
+		return 0;
+
+	acc = ua_account(ua);
+	regint = pl_u32(&word[0]);
+
+	err = account_set_regint(acc, regint);
+	if (err)
+		return err;
+
+	if (regint) {
+		re_hprintf(pf, "registering %s with interval %u seconds\n",
+			   account_aor(acc), regint);
+		err = ua_register(ua);
+		if (err)
+			return err;
+	}
+	else {
+		re_hprintf(pf, "unregistering %s\n", account_aor(acc));
+		ua_unregister(ua);
+	}
+
+	return 0;
+}
+
+
+static int cmd_addheader(struct re_printf *pf, void *arg)
+{
+	const struct cmd_arg *carg = arg;
+	struct pl w1, w2;
+	struct ua *ua = menu_ua_carg(pf, carg, &w1, &w2);
+	const char *usage = "usage: /uaaddheader <key>=<value> <ua-idx>\n";
+	struct pl n, v;
+	int err;
+	int ret;
+
+	if (!ua) {
+		re_hprintf(pf, usage);
+		return EINVAL;
+	}
+
+	err = re_regex(w1.p, w1.l, "[^=]+=[~]+", &n, &v);
+	if (err) {
+		re_hprintf(pf, "invalid key value pair %r\n", &w1);
+		re_hprintf(pf, usage);
+		return EINVAL;
+	}
+
+	struct mbuf mbe;
+	mbuf_init(&mbe);
+	ret = mbuf_printf(&mbe, "%H", uri_header_unescape, &v);
+	if (ret == 0) {
+		v.p = (const char *)mbe.buf;
+		v.l = mbe.end;
+	}
+
+	ret = ua_add_custom_hdr(ua, &n, &v);
+	mem_deref(mbe.buf);
+	return ret;
+}
+
+
+static int cmd_rmheader(struct re_printf *pf, void *arg)
+{
+	const struct cmd_arg *carg = arg;
+	struct pl w1, w2;
+	struct ua *ua = menu_ua_carg(pf, carg, &w1, &w2);
+	const char *usage = "usage: /uarmheader <key> <ua-idx>\n";
+	struct pl n;
+	int err;
+
+	if (!ua) {
+		re_hprintf(pf, usage);
+		return EINVAL;
+	}
+
+	err = re_regex(w1.p, w1.l, "[^ ]*", &n);
+	if (err) {
+		re_hprintf(pf, "invalid key %r\n", &w1);
+		re_hprintf(pf, usage);
+		return EINVAL;
+	}
+
+	return ua_rm_custom_hdr(ua, &n);
+}
+
+
 static int switch_video_source(struct re_printf *pf, void *arg)
 {
 	const struct cmd_arg *carg = arg;
@@ -1212,6 +1475,7 @@ static int cmd_tls_subject(struct re_printf *pf, void *unused)
 /*Static call menu*/
 static const struct cmd cmdv[] = {
 
+{"100rel"    ,0,    CMD_PRM, "Set 100rel mode",         cmd_set_100rel_mode  },
 {"about",     0,          0, "About box",               about_box            },
 {"accept",    'a',        0, "Accept incoming call",    cmd_answer           },
 {"acceptdir", 0,    CMD_PRM, "Accept incoming call with audio and video"
@@ -1224,19 +1488,26 @@ static const struct cmd cmdv[] = {
 {"dialdir",   0,    CMD_PRM, "Dial with audio and video"
                              "direction.",              cmd_dialdir          },
 {"dnd",       0,    CMD_PRM, "Set Do not Disturb",      cmd_dnd              },
+{"entransp",  0,    CMD_PRM, "Enable/Disable transport", cmd_enable_transp   },
 {"hangup",    'b',        0, "Hangup call",             cmd_hangup           },
 {"hangupall", 0,    CMD_PRM, "Hangup all calls with direction"
                                                        ,cmd_hangupall        },
 {"help",      'h',        0, "Help menu",               print_commands       },
 {"listcalls", 'l',        0, "List active calls",       cmd_print_calls      },
 {"options",   'o',  CMD_PRM, "Options",                 options_command      },
+{"refer",     'R',  CMD_PRM, "Send REFER outside dialog", cmd_refer          },
 {"reginfo",   'r',        0, "Registration info",       ua_print_reg_status  },
 {"setadelay", 0,    CMD_PRM, "Set answer delay for outgoing call",
                                                         cmd_set_adelay       },
+{"setansval", 0,    CMD_PRM, "Set value for Call-Info/Alert-Info",
+                                                        cmd_set_ansval       },
 {"uadel",     0,    CMD_PRM, "Delete User-Agent",       cmd_ua_delete        },
 {"uadelall",  0,    CMD_PRM, "Delete all User-Agents",  cmd_ua_delete_all    },
 {"uafind",    0,    CMD_PRM, "Find User-Agent <aor>",   cmd_ua_find          },
 {"uanew",     0,    CMD_PRM, "Create User-Agent",       create_ua            },
+{"uareg",     0,    CMD_PRM, "UA register <regint> [index]", cmd_uareg       },
+{"uaaddheader", 0,  CMD_PRM, "Add custom header to UA",      cmd_addheader   },
+{"uarmheader",  0,  CMD_PRM, "Remove custom header from UA", cmd_rmheader    },
 {"vidsrc",    0,    CMD_PRM, "Switch video source",     switch_video_source  },
 {NULL,        KEYCODE_ESC,0, "Hangup call",             cmd_hangup           },
 
@@ -1271,7 +1542,7 @@ static const struct cmd dialcmdv[] = {
  */
 int static_menu_register(void)
 {
-	return cmd_register(baresip_commands(), cmdv, ARRAY_SIZE(cmdv));
+	return cmd_register(baresip_commands(), cmdv, RE_ARRAY_SIZE(cmdv));
 }
 
 
@@ -1295,7 +1566,7 @@ int dial_menu_register(void)
 
 	if (!cmds_find(baresip_cmd, dialcmdv))
 		return cmd_register(baresip_cmd,
-			dialcmdv, ARRAY_SIZE(dialcmdv));
+			dialcmdv, RE_ARRAY_SIZE(dialcmdv));
 
 	return 0;
 }
