@@ -25,7 +25,7 @@
  * stream. The audio source is not faded.
  */
 
-#define DEFAULT_FADE_PACKETS 8
+#define DEFAULT_FADE_TIME 160  /*default fading time in ms*/
 
 /**
  * State machine. See:
@@ -85,8 +85,9 @@ struct mixstatus {
 	float ausvol;                   /**< Volume for mixed audio source   */
 	size_t sampc;                   /**< Stream sample count for frame   */
 	size_t nbytes;                  /**< Stream bytes per frame          */
-	size_t i_fade;                  /**< Fade-in/-out counter            */
-	size_t n_fade;                  /**< Fade-in/-out steps              */
+	uint16_t i_fade;                /**< Fade-in/-out counter            */
+	uint16_t n_fade;                /**< Fade-in/-out steps              */
+	float delta_fade;               /**< linear delta accumulation       */
 	struct aufilt_prm prm;          /**< Audio filter parameter          */
 
 	struct auresamp resamp;         /**< Optional audio resampler        */
@@ -127,10 +128,9 @@ static int init_aubuf(struct mixstatus *st)
 	uint32_t v = 2;
 	size_t maxsz;
 	size_t wishsz;
-	size_t num_bytes = st->sampc * aufmt_sample_size(st->prm.fmt);
 
 	conf_get_u32(conf_cur(), "mixausrc_wish_size", &v);
-	wishsz = v * num_bytes;
+	wishsz = v * st->nbytes;
 	maxsz = 2 * wishsz;
 
 	st->aubuf = mem_deref(st->aubuf);
@@ -143,10 +143,11 @@ static int init_aubuf(struct mixstatus *st)
 		goto out;
 	}
 
+	aubuf_set_live(st->aubuf, false);
 	if (st->rbuf)
 		return 0;
 
-	st->rbuf = mem_zalloc(sizeof(int16_t)*st->sampc, NULL);
+	st->rbuf = mem_zalloc(st->nbytes, NULL);
 	if (!st->rbuf) {
 		warning("mixausrc: Could not allocate rbuf.\n");
 		goto out;
@@ -157,13 +158,14 @@ out:
 }
 
 
-static int process_resamp(struct mixstatus *st, const struct auframe *af)
+static int process_resamp(struct mixstatus *st, const struct auframe *afsrc)
 {
 	int err = 0;
-	size_t nbytes = auframe_size(af);
-	size_t sampc = st->sampc;
-	if (af->fmt != AUFMT_S16LE)
+	if (afsrc->fmt != AUFMT_S16LE) {
+		warning("mixausrc: sample format %s not supported\n",
+			aufmt_name(afsrc->fmt));
 		return EINVAL;
+	}
 
 	if (!st->resamp.resample) {
 		debug("mixausrc: resample ausrc %u/%u -> %u/%u\n",
@@ -179,10 +181,7 @@ static int process_resamp(struct mixstatus *st, const struct auframe *af)
 		}
 
 		st->sampvrs = mem_deref(st->sampvrs);
-		if (st->nbytes > nbytes)
-			nbytes = st->nbytes;
-
-		st->sampvrs = mem_zalloc(nbytes, NULL);
+		st->sampvrs = mem_zalloc(st->nbytes, NULL);
 		if (!st->sampvrs) {
 			warning("mixausrc: could not alloc resample buffer\n");
 			return ENOMEM;
@@ -190,11 +189,10 @@ static int process_resamp(struct mixstatus *st, const struct auframe *af)
 	}
 
 	if (st->resamp.resample) {
-		if (af->sampc > sampc)
-			sampc = af->sampc;
-
+		size_t sampc = st->sampc;
 		err = auresamp(&st->resamp, st->sampvrs, &sampc,
-			       af->sampv, af->sampc);
+			       afsrc->sampv, afsrc->sampc);
+
 		if (sampc != st->sampc) {
 			warning("mixausrc: unexpected sample count "
 					"%u vs. %u\n", sampc, st->sampc);
@@ -205,29 +203,64 @@ static int process_resamp(struct mixstatus *st, const struct auframe *af)
 	if (err)
 		warning("mixausrc: could not resample frame (%m)\n", err);
 
-
 	return err;
 }
 
 
-static void ausrc_read_handler(struct auframe *af, void *arg)
+static void ausrc_prm_af(struct ausrc_prm *ausprm, struct auframe *afsrc)
+{
+	ausprm->srate = afsrc->srate;
+	ausprm->ch    = afsrc->ch;
+	ausprm->fmt   = afsrc->fmt;
+}
+
+
+static void ausrc_prm_aufilt(struct ausrc_prm *ausprm,
+			     struct aufilt_prm *filprm)
+{
+	ausprm->srate = filprm->srate;
+	ausprm->ch    = filprm->ch;
+	ausprm->fmt   = filprm->fmt;
+}
+
+
+static void ausrc_read_handler(struct auframe *afsrc, void *arg)
 {
 	struct mixstatus *st = arg;
-	size_t num_bytes;
 	int err = 0;
 
+	if (!st->prm.srate || !st->prm.ch)
+		return;
+
+	ausrc_prm_af(&st->ausrc_prm, afsrc);
+	if (!st->ausrc_prm.srate || !st->ausrc_prm.ch)
+		return;
+
+	if (!st->sampc || !st->nbytes)
+		return;
+
 	if (st->ausrc_prm.srate != st->prm.srate ||
 			st->ausrc_prm.ch != st->prm.ch)
-		err = process_resamp(st, af);
+		err = process_resamp(st, afsrc);
 
-	if (err)
+	if (err) {
 		st->nextmode = FM_FADEIN;
+		return;
+	}
 
-	num_bytes = st->sampc * aufmt_sample_size(st->prm.fmt);
-	if (st->sampvrs)
-		aubuf_write(st->aubuf, st->sampvrs, num_bytes);
+	if (!st->aubuf) {
+		err = init_aubuf(st);
+		if (err) {
+			st->nextmode = FM_FADEIN;
+			return;
+		}
+	}
+
+	if (st->sampvrs) {
+		aubuf_write(st->aubuf, st->sampvrs, st->nbytes);
+	}
 	else
-		aubuf_write(st->aubuf, af->sampv, num_bytes);
+		aubuf_write(st->aubuf, afsrc->sampv, st->nbytes);
 }
 
 
@@ -246,14 +279,10 @@ static int start_ausrc(struct mixstatus *st)
 {
 	int err;
 
-	err = init_aubuf(st);
-	if (err)
-		goto out;
-
 	auresamp_init(&st->resamp);
-	err = ausrc_alloc(&st->ausrc, baresip_ausrcl(), NULL, st->module,
-			&st->ausrc_prm, st->param, ausrc_read_handler,
-			ausrc_error_handler, st);
+	err = ausrc_alloc(&st->ausrc, baresip_ausrcl(), st->module,
+			  &st->ausrc_prm, st->param, ausrc_read_handler,
+			  ausrc_error_handler, st);
 
 	if (!st->ausrc) {
 		warning("mixausrc: Could not start audio source %s with "
@@ -262,7 +291,6 @@ static int start_ausrc(struct mixstatus *st)
 		st->mode = FM_FADEIN;
 	}
 
-out:
 	/* now we are ready for next start_process */
 	st->module = mem_deref(st->module);
 	st->param = mem_deref(st->param);
@@ -275,7 +303,10 @@ static int stop_ausrc(struct mixstatus *st)
 {
 	st->ausrc   = mem_deref(st->ausrc);
 	st->aubuf   = mem_deref(st->aubuf);
+	st->rbuf    = mem_deref(st->rbuf);
 	st->sampvrs = mem_deref(st->sampvrs);
+	st->sampc   = 0;
+	st->nbytes  = 0;
 	st->aubuf_started = false;
 	return 0;
 }
@@ -283,12 +314,9 @@ static int stop_ausrc(struct mixstatus *st)
 
 static void destruct(struct mixstatus *st)
 {
+	stop_ausrc(st);
 	mem_deref(st->module);
 	mem_deref(st->param);
-	mem_deref(st->ausrc);
-	mem_deref(st->aubuf);
-	mem_deref(st->rbuf);
-	mem_deref(st->sampvrs);
 }
 
 
@@ -319,6 +347,8 @@ static void mixstatus_init(struct mixstatus *st, struct aufilt_prm *prm)
 	st->minvol = 1.;
 	st->ausvol = 1.;
 	st->i_fade = 0;
+
+	/* initialize with configured values */
 	st->prm = *prm;
 	st->ausrc_prm.ch = prm->ch;
 	st->ausrc_prm.fmt = prm->fmt;
@@ -333,6 +363,7 @@ static int encode_update(struct aufilt_enc_st **stp, void **ctx,
 	struct mixausrc_enc *st;
 	(void)au;
 	(void)af;
+	(void)prm;
 
 	if (!stp || !ctx || !prm)
 		return EINVAL;
@@ -384,42 +415,47 @@ static int decode_update(struct aufilt_dec_st **stp, void **ctx,
 
 
 /**
- * Fades a float: fade(0, n) = v0 to fade(n, n) = 1.
+ * Fade-In:  values from st->minvol to 1.0
+ * Fade-Out: values form 1.0 to st->minvol
+ *
+ * @param st   mixausrc status object
+ * @param dir  fading direction (fade-in / fade-out)
+ *
+ * @return float sample multiplication factor
  */
-static float fade(size_t i, size_t n, float v0)
+static float fade_linear(struct mixstatus *st, enum mixmode dir)
 {
-	float x;
-	float r;
+	float factor = st->i_fade * st->delta_fade;
+	++st->i_fade;
 
-	if (v0==1.)
-		return 1.;
-
-	x = ((float) (i-n/2)) / (n/32);
-	r = .5 + x / (2*sqrt(1+x*x));
-
-	return v0 + (1. - v0) * (1. - r);
+	if (dir == FM_FADEIN)
+		return (st->minvol + factor) > 1. ? 1. : st->minvol + factor;
+	else
+		return (1. - factor) < st->minvol ? st->minvol : 1. - factor;
 }
 
 
-static void fade_int16(struct mixstatus *st, int16_t *data, size_t n,
-		       int dir)
+static void fade_int16(struct mixstatus *st, int16_t *data, uint16_t n,
+	enum mixmode dir)
 {
-	size_t i;
-	for (i = 0; i < n; i++)
-		data[i] *= fade(st->i_fade + i*dir, st->n_fade, st->minvol);
+	uint16_t i;
+	for (i = 0; (i < n) && (st->i_fade < st->n_fade); ++i) {
+		data[i] *= fade_linear(st, dir);
+	}
 }
 
 
-static void fade_float(struct mixstatus *st, float *data, size_t n,
-		       int dir)
+static void fade_float(struct mixstatus *st, float *data, uint16_t n,
+	enum mixmode dir)
 {
-	size_t i;
-	for (i = 0; i < n; i++)
-		data[i] *= fade(st->i_fade + i*dir, st->n_fade, st->minvol);
+	uint16_t i;
+	for (i = 0; (i < n) && (st->i_fade < st->n_fade); ++i)
+		data[i] *= fade_linear(st, dir);
 }
 
 
-static int fadeframe(struct mixstatus *st, struct auframe *af, int dir)
+static int fadeframe(struct mixstatus *st, struct auframe *af,
+	enum mixmode dir)
 {
 	if (af->fmt == AUFMT_S16LE)
 		fade_int16(st, af->sampv, af->sampc, dir);
@@ -432,6 +468,35 @@ static int fadeframe(struct mixstatus *st, struct auframe *af, int dir)
 }
 
 
+static void clear_int16(struct mixstatus *st, int16_t *data, uint16_t n)
+{
+	uint16_t i;
+	for (i = 0; i < n; ++i)
+		data[i] *= st->minvol;
+}
+
+
+static void clear_float(struct mixstatus *st, float *data, uint16_t n)
+{
+	uint16_t i;
+	for (i = 0; i < n; ++i)
+		data[i] *= st->minvol;
+}
+
+
+static int clear_frame(struct mixstatus *st, struct auframe *af)
+{
+	if (af->fmt == AUFMT_S16LE)
+		clear_int16(st, af->sampv, af->sampc);
+	else if (af->fmt == AUFMT_FLOAT)
+		clear_float(st, af->sampv, af->sampc);
+	else
+		return EINVAL;
+
+	return 0;
+}
+
+
 static void mix_int16(struct mixstatus *st, int16_t *data, size_t n)
 {
 	size_t i;
@@ -463,28 +528,35 @@ static int mixframe(struct mixstatus *st, struct auframe *af)
 }
 
 
+static void aufilt_prm_update(struct aufilt_prm *prm, struct auframe *af)
+{
+	prm->srate = af->srate;
+	prm->ch    = af->ch;
+	prm->fmt   = af->fmt;
+}
+
+
 static int process(struct mixstatus *st, struct auframe *af)
 {
 	size_t n = af->sampc;
-	uint32_t pnbr = DEFAULT_FADE_PACKETS;
 	int err = 0;
 
 	st->nbytes = auframe_size(af);
 	if (!st->sampc) {
 		st->sampc = n;
 		st->ausrc_prm.ptime = (uint32_t) n * 1000 /
-			st->ausrc_prm.srate / st->ausrc_prm.ch;
-
-		conf_get_u32(conf_cur(), "mixausrc_fade_packets", &pnbr);
-		st->n_fade = n * pnbr;
+			(af->srate * af->ch);
 	}
 	else if (st->sampc != n) {
 		warning("mixausrc: sampc changed %lu --> %lu.\n",
-				st->sampc, n);
+			st->sampc, n);
 		stop_ausrc(st);
+		st->nextmode = FM_FADEIN;
+		st->sampc = 0;
 		return EINVAL;
 	}
 
+	aufilt_prm_update(&st->prm, af);
 	if (st->mode == FM_FADEOUT && st->i_fade == st->n_fade)
 		st->mode = FM_MIX;
 
@@ -499,7 +571,7 @@ static int process(struct mixstatus *st, struct auframe *af)
 		/* a command was invoked */
 		/* process nextmode */
 		if (st->mode != st->nextmode) {
-			debug("mixausrc: mode %s --> %s",
+			debug("mixausrc: mode %s --> %s\n",
 					str_mixmode(st->mode),
 					str_mixmode(st->nextmode));
 			if (st->mode == FM_MIX)
@@ -512,20 +584,18 @@ static int process(struct mixstatus *st, struct auframe *af)
 
 	switch (st->mode) {
 	case FM_FADEIN: {
-		err = fadeframe(st, af, -1);
-		st->i_fade -= n;
-		if (st->i_fade <= 0) {
+		err = fadeframe(st, af, st->mode);
+		if (st->i_fade >= st->n_fade) {
 			st->i_fade = 0;
 			st->mode = FM_IDLE;
 		}
 	}
 	break;
 	case FM_FADEOUT: {
-		 err = fadeframe(st, af, +1);
-		 st->i_fade += n;
-		 if (st->i_fade >= st->n_fade) {
-			 st->i_fade = st->n_fade;
-			 st->mode = FM_MIX;
+		err = fadeframe(st, af, st->mode);
+		if (st->i_fade >= st->n_fade) {
+			st->i_fade = 0;
+			st->mode = FM_MIX;
 		}
 	}
 	break;
@@ -538,6 +608,10 @@ static int process(struct mixstatus *st, struct auframe *af)
 		}
 		else if (!st->ausrc) {
 			start_ausrc(st);
+			clear_frame(st, af);
+		}
+		else {
+			clear_frame(st, af);
 		}
 	}
 	break;
@@ -630,9 +704,15 @@ static int start_process(struct mixstatus* st, const char *name,
 	if (err)
 		return err;
 
+	/* Fading arguments */
 	st->minvol = pl_isset(&pl3) ? conv_volume(&pl3) : 0.;
 	st->ausvol = pl_isset(&pl4) ? conv_volume(&pl4) : 1.;
+	st->i_fade = 0;
+	st->n_fade = (DEFAULT_FADE_TIME * st->ausrc_prm.srate) / 1000;
+	st->delta_fade = (1.0 - st->minvol) / st->n_fade;
 
+	stop_ausrc(st);
+	ausrc_prm_aufilt(&st->ausrc_prm, &st->prm);
 	st->nextmode = FM_FADEOUT;
 
 	return err;
@@ -789,7 +869,7 @@ static int module_init(void)
 	aufilt_register(baresip_aufiltl(), &mixausrc);
 
 	/* register commands */
-	err  = cmd_register(baresip_commands(), cmdv, ARRAY_SIZE(cmdv));
+	err  = cmd_register(baresip_commands(), cmdv, RE_ARRAY_SIZE(cmdv));
 	return err;
 }
 
