@@ -30,8 +30,7 @@ struct ausrc_st {
 	ausrc_read_h *rh;
 	void *arg;
 	volatile bool ready;
-	unsigned ch;
-	enum aufmt fmt;
+	struct ausrc_prm prm;
 };
 
 struct auplay_st {
@@ -39,8 +38,7 @@ struct auplay_st {
 	auplay_write_h *wh;
 	void *arg;
 	volatile bool ready;
-	unsigned ch;
-	enum aufmt fmt;
+	struct auplay_prm prm;
 };
 
 
@@ -69,11 +67,10 @@ static int read_callback(const void *inputBuffer, void *outputBuffer,
 	if (!st->ready)
 		return paAbort;
 
-	sampc = frameCount * st->ch;
+	sampc = frameCount * st->prm.ch;
 
-	af.fmt   = st->fmt;
-	af.sampv = (void *)inputBuffer;
-	af.sampc = sampc;
+	auframe_init(&af, st->prm.fmt, (void *)inputBuffer, sampc,
+		     st->prm.srate, st->prm.ch);
 	af.timestamp = Pa_GetStreamTime(st->stream_rd) * AUDIO_TIMEBASE;
 
 	st->rh(&af, st->arg);
@@ -98,9 +95,10 @@ static int write_callback(const void *inputBuffer, void *outputBuffer,
 	if (!st->ready)
 		return paAbort;
 
-	sampc = frameCount * st->ch;
+	sampc = frameCount * st->prm.ch;
 
-	auframe_init(&af, st->fmt, outputBuffer, sampc);
+	auframe_init(&af, st->prm.fmt, outputBuffer, sampc, st->prm.srate,
+		     st->prm.ch);
 
 	st->wh(&af, st->arg);
 
@@ -212,7 +210,6 @@ static void auplay_destructor(void *arg)
 
 
 static int src_alloc(struct ausrc_st **stp, const struct ausrc *as,
-		     struct media_ctx **ctx,
 		     struct ausrc_prm *prm, const char *device,
 		     ausrc_read_h *rh, ausrc_error_h *errh, void *arg)
 {
@@ -220,7 +217,6 @@ static int src_alloc(struct ausrc_st **stp, const struct ausrc *as,
 	PaDeviceIndex dev_index;
 	int err;
 
-	(void)ctx;
 	(void)device;
 	(void)errh;
 
@@ -238,8 +234,7 @@ static int src_alloc(struct ausrc_st **stp, const struct ausrc *as,
 
 	st->rh  = rh;
 	st->arg = arg;
-	st->ch  = prm->ch;
-	st->fmt = prm->fmt;
+	st->prm = *prm;
 
 	st->ready = true;
 
@@ -281,8 +276,7 @@ static int play_alloc(struct auplay_st **stp, const struct auplay *ap,
 
 	st->wh  = wh;
 	st->arg = arg;
-	st->ch  = prm->ch;
-	st->fmt = prm->fmt;
+	st->prm  = *prm;
 
 	st->ready = true;
 
@@ -305,34 +299,78 @@ static int pa_init(void)
 	PaError paerr;
 	int i, n, err = 0;
 
-	paerr = Pa_Initialize();
+	if (log_level_get() == LEVEL_DEBUG) {
+		paerr = Pa_Initialize();
+	}
+	else {
+		fs_stdio_hide();
+		paerr = Pa_Initialize();
+		fs_stdio_restore();
+	}
 	if (paNoError != paerr) {
 		warning("portaudio: init: %s\n", Pa_GetErrorText(paerr));
 		return ENODEV;
 	}
 
+	if (paNoDevice != Pa_GetDefaultInputDevice())
+		err |= ausrc_register(&ausrc, baresip_ausrcl(), "portaudio",
+				      src_alloc);
+
+	if (paNoDevice != Pa_GetDefaultOutputDevice())
+		err |= auplay_register(&auplay, baresip_auplayl(), "portaudio",
+				       play_alloc);
+
+	if (err)
+		return err;
+
 	n = Pa_GetDeviceCount();
 
 	info("portaudio: device count is %d\n", n);
 
-	for (i=0; i<n; i++) {
-		const PaDeviceInfo *devinfo;
-
-		devinfo = Pa_GetDeviceInfo(i);
+	for (i = 0; i < n; i++) {
+		const PaDeviceInfo *devinfo = Pa_GetDeviceInfo(i);
+		struct mediadev *dev;
 
 		debug("portaudio: device %d: %s\n", i, devinfo->name);
-		(void)devinfo;
+
+		if (devinfo->maxInputChannels > 0) {
+			err = mediadev_add(&ausrc->dev_list, devinfo->name);
+			if (err) {
+				warning("portaudio: mediadev err %m\n", err);
+				return err;
+			}
+
+			dev = mediadev_find(&ausrc->dev_list, devinfo->name);
+			if (!dev)
+				continue;
+
+			dev->host_index	  = devinfo->hostApi;
+			dev->device_index = i;
+			dev->src.is_default =
+				(i == Pa_GetDefaultInputDevice());
+			dev->src.channels = devinfo->maxInputChannels;
+		}
+
+		if (devinfo->maxOutputChannels > 0) {
+			mediadev_add(&auplay->dev_list, devinfo->name);
+			if (err) {
+				warning("portaudio: mediadev err %m\n", err);
+				return err;
+			}
+
+			dev = mediadev_find(&auplay->dev_list, devinfo->name);
+			if (!dev)
+				continue;
+
+			dev->host_index	  = devinfo->hostApi;
+			dev->device_index = i;
+			dev->play.is_default =
+				(i == Pa_GetDefaultOutputDevice());
+			dev->play.channels = devinfo->maxOutputChannels;
+		}
 	}
 
-	if (paNoDevice != Pa_GetDefaultInputDevice())
-		err |= ausrc_register(&ausrc, baresip_ausrcl(),
-				      "portaudio", src_alloc);
-
-	if (paNoDevice != Pa_GetDefaultOutputDevice())
-		err |= auplay_register(&auplay, baresip_auplayl(),
-				       "portaudio", play_alloc);
-
-	return err;
+	return 0;
 }
 
 
