@@ -4,24 +4,24 @@
  * Copyright (C) 2010 Alfred E. Heggestad
  */
 
-
-#include <pthread.h>
-#include "modules/audio_processing/aec/echo_cancellation.h"
+#define WEBRTC_POSIX 1
+#include <webrtc/modules/audio_processing/include/audio_processing.h>
+#include <webrtc/system_wrappers/include/trace.h>
 
 
 #define MAX_CHANNELS         1
+#define BLOCKSIZE           10  /* ms */
 
 
 using namespace webrtc;
 
 
 struct aec {
-	AecConfig config;
-	void *inst;
-	pthread_mutex_t mutex;
+	AudioProcessing *inst;
+	mtx_t mutex;
 	uint32_t srate;
-	uint32_t subframe_len;
-	int num_bands;
+	uint8_t ch;
+	uint32_t blocksize;
 };
 
 
@@ -44,4 +44,3 @@ int webrtc_aec_decode(struct aufilt_dec_st *st, struct auframe *af);
 /* Common */
 
 int  webrtc_aec_alloc(struct aec **stp, void **ctx, struct aufilt_prm *prm);
-void webrtc_aec_debug(const struct aec *aec);
