@@ -23,7 +23,7 @@ struct ausrc_st {
 	volatile bool rdy;
 	size_t inuse;
 	size_t sampsz;
-	enum aufmt fmt;
+	struct ausrc_prm prm;
 	ausrc_read_h *rh;
 	void *arg;
 };
@@ -106,9 +106,9 @@ static void CALLBACK waveInCallback(HWAVEOUT hwo,
 		if (st->inuse < (READ_BUFFERS-1))
 			add_wave_in(st);
 
-		af.fmt   = st->fmt;
-		af.sampv = (void *)wh->lpData;
-		af.sampc = wh->dwBytesRecorded/st->sampsz;
+		auframe_init(&af, st->prm.fmt, (void *)wh->lpData,
+			     wh->dwBytesRecorded / st->sampsz, st->prm.srate,
+			     st->prm.ch);
 		af.timestamp = tmr_jiffies_usec();
 
 		st->rh(&af, st->arg);
@@ -145,7 +145,7 @@ static int read_stream_open(struct ausrc_st *st, const struct ausrc_prm *prm,
 	st->wavein = NULL;
 	st->pos = 0;
 	st->rdy = false;
-	st->fmt = prm->fmt;
+	st->prm = *prm;
 
 	sampc = prm->srate * prm->ch * prm->ptime / 1000;
 
@@ -160,7 +160,7 @@ static int read_stream_open(struct ausrc_st *st, const struct ausrc_prm *prm,
 	wfmt.nChannels       = prm->ch;
 	wfmt.nSamplesPerSec  = prm->srate;
 	wfmt.wBitsPerSample  = (WORD)(st->sampsz * 8);
-	wfmt.nBlockAlign     = prm->ch * st->sampsz;
+	wfmt.nBlockAlign     = (WORD)(prm->ch * st->sampsz);
 	wfmt.nAvgBytesPerSec = wfmt.nSamplesPerSec * wfmt.nBlockAlign;
 	wfmt.cbSize          = 0;
 
@@ -214,7 +214,6 @@ static int find_dev(const char *name, unsigned int *dev)
 
 
 int winwave_src_alloc(struct ausrc_st **stp, const struct ausrc *as,
-		      struct media_ctx **ctx,
 		      struct ausrc_prm *prm, const char *device,
 		      ausrc_read_h *rh, ausrc_error_h *errh, void *arg)
 {
@@ -222,7 +221,6 @@ int winwave_src_alloc(struct ausrc_st **stp, const struct ausrc *as,
 	int err;
 	unsigned int dev;
 
-	(void)ctx;
 	(void)errh;
 
 	if (!stp || !as || !prm)
