@@ -24,7 +24,7 @@ static void destructor(void *arg)
 	list_clear(&acc->vidcodecl);
 	mem_deref(acc->auth_user);
 	mem_deref(acc->auth_pass);
-	for (i=0; i<ARRAY_SIZE(acc->outboundv); i++)
+	for (i=0; i<RE_ARRAY_SIZE(acc->outboundv); i++)
 		mem_deref(acc->outboundv[i]);
 	mem_deref(acc->regq);
 	mem_deref(acc->sipnat);
@@ -40,7 +40,10 @@ static void destructor(void *arg)
 	mem_deref(acc->ausrc_dev);
 	mem_deref(acc->auplay_mod);
 	mem_deref(acc->auplay_dev);
+	mem_deref(acc->cert);
 	mem_deref(acc->extra);
+	mem_deref(acc->uas_user);
+	mem_deref(acc->uas_pass);
 }
 
 
@@ -68,6 +71,17 @@ static int param_u32(uint32_t *v, const struct pl *params, const char *name)
 }
 
 
+static int param_bool(bool *v, const struct pl *params, const char *name)
+{
+	struct pl pl;
+
+	if (msg_param_decode(params, name, &pl))
+		return 0;
+
+	return pl_bool(v, &pl);
+}
+
+
 /*
  * Decode STUN parameters, inspired by RFC 7064
  *
@@ -80,19 +94,27 @@ static int param_u32(uint32_t *v, const struct pl *params, const char *name)
 static int stunsrv_decode(struct account *acc, const struct sip_addr *aor)
 {
 	struct pl srv, tmp;
+	struct uri uri;
 	int err;
 
 	if (!acc || !aor)
 		return EINVAL;
 
+	memset(&uri, 0, sizeof(uri));
 	if (0 == msg_param_decode(&aor->params, "stunserver", &srv)) {
 
 		info("using stunserver: '%r'\n", &srv);
 
-		err = stunuri_decode(&acc->stun_host, &srv);
+		err = uri_decode(&uri, &srv);
 		if (err) {
-			warning("account: decode '%r' failed: %m\n",
+			warning("account: decode '%r' failed (%m)\n",
 				&srv, err);
+			return err;
+		}
+
+		err = stunuri_decode_uri(&acc->stun_host, &uri);
+		if (err) {
+			return err;
 		}
 	}
 
@@ -100,9 +122,15 @@ static int stunsrv_decode(struct account *acc, const struct sip_addr *aor)
 
 	if (0 == msg_param_exists(&aor->params, "stunuser", &tmp))
 		err |= param_dstr(&acc->stun_user, &aor->params, "stunuser");
+	else if (pl_isset(&uri.user))
+		err |= re_sdprintf(&acc->stun_user, "%H",
+					uri_user_unescape, &uri.user);
 
 	if (0 == msg_param_exists(&aor->params, "stunpass", &tmp))
 		err |= param_dstr(&acc->stun_pass, &aor->params, "stunpass");
+	else if (pl_isset(&uri.password))
+		err |= re_sdprintf(&acc->stun_pass, "%H",
+					uri_password_unescape, &uri.password);
 
 	return err;
 }
@@ -116,9 +144,35 @@ static int media_decode(struct account *acc, const struct pl *prm)
 	if (!acc || !prm)
 		return EINVAL;
 
-	err |= param_dstr(&acc->mencid,  prm, "mediaenc");
-	err |= param_dstr(&acc->mnatid,  prm, "medianat");
-	err |= param_u32(&acc->ptime,    prm, "ptime"   );
+	err |= param_dstr(&acc->mencid,   prm, "mediaenc");
+	err |= param_dstr(&acc->mnatid,   prm, "medianat");
+	err |= param_u32(&acc->ptime,     prm, "ptime");
+	err |= param_bool(&acc->rtcp_mux, prm, "rtcp_mux");
+	err |= param_bool(&acc->pinhole, prm,  "natpinhole");
+
+	return err;
+}
+
+
+/* Decode cert parameter */
+static int cert_decode(struct account *acc, const struct pl *prm)
+{
+	int err = 0;
+
+	if (!acc || !prm)
+		return EINVAL;
+
+	err = param_dstr(&acc->cert, prm, "cert");
+	if (err)
+		return err;
+
+	if (!str_isset(acc->cert))
+		return 0;
+
+	if (!fs_isfile(acc->cert)) {
+		warning("account: certificate %s not found\n", acc->cert);
+		err = errno;
+	}
 
 	return err;
 }
@@ -176,6 +230,12 @@ static void answermode_decode(struct account *prm, const struct pl *pl)
 		else if (0 == pl_strcasecmp(&amode, "early")) {
 			prm->answermode = ANSWERMODE_EARLY;
 		}
+		else if (0 == pl_strcasecmp(&amode, "early-video")) {
+			prm->answermode = ANSWERMODE_EARLY_VIDEO;
+		}
+		else if (0 == pl_strcasecmp(&amode, "early-audio")) {
+			prm->answermode = ANSWERMODE_EARLY_AUDIO;
+		}
 		else if (0 == pl_strcasecmp(&amode, "auto")) {
 			prm->answermode = ANSWERMODE_AUTO;
 		}
@@ -189,7 +249,31 @@ static void answermode_decode(struct account *prm, const struct pl *pl)
 }
 
 
-static void autoanswer_allow_decode(struct account *prm, const struct pl *pl)
+static void rel100_decode(struct account *prm, const struct pl *pl)
+{
+	struct pl rmode;
+
+	prm->rel100_mode = REL100_DISABLED;
+
+	if (0 == msg_param_decode(pl, "100rel", &rmode)) {
+
+		if (0 == pl_strcasecmp(&rmode, "no")) {
+			prm->rel100_mode = REL100_DISABLED;
+		}
+		else if (0 == pl_strcasecmp(&rmode, "yes")) {
+			prm->rel100_mode = REL100_ENABLED;
+		}
+		else if (0 == pl_strcasecmp(&rmode, "required")) {
+			prm->rel100_mode = REL100_REQUIRED;
+		}
+		else {
+			warning("account: 100rel mode unknown (%r)\n", &rmode);
+		}
+	}
+}
+
+
+static void autoanswer_decode(struct account *prm, const struct pl *pl)
 {
 	struct pl v;
 
@@ -198,6 +282,18 @@ static void autoanswer_allow_decode(struct account *prm, const struct pl *pl)
 			prm->sipans = true;
 		}
 	}
+
+	if (0 == msg_param_decode(pl, "sip_autoanswer_beep", &v)) {
+		if (0 == pl_strcasecmp(&v, "on")) {
+			prm->sipansbeep = SIPANSBEEP_ON;
+		}
+		else if (0 == pl_strcasecmp(&v, "off")) {
+			prm->sipansbeep = SIPANSBEEP_OFF;
+		}
+		else if (0 == pl_strcasecmp(&v, "local")) {
+			prm->sipansbeep = SIPANSBEEP_LOCAL;
+		}
+	}
 }
 
 
@@ -211,6 +307,9 @@ static void dtmfmode_decode(struct account *prm, const struct pl *pl)
 		if (0 == pl_strcasecmp(&dtmfmode, "info")) {
 			prm->dtmfmode = DTMFMODE_SIP_INFO;
 		}
+		else if (0 == pl_strcasecmp(&dtmfmode, "auto")) {
+			prm->dtmfmode = DTMFMODE_AUTO;
+		}
 		else {
 			prm->dtmfmode = DTMFMODE_RTP_EVENT;
 		}
@@ -218,6 +317,28 @@ static void dtmfmode_decode(struct account *prm, const struct pl *pl)
 }
 
 
+static void inreq_mode_decode(struct account *prm, const struct pl *pl)
+{
+	struct pl mode;
+
+	prm->inreq_mode = INREQ_MODE_ON;
+
+	if (0 == msg_param_decode(pl, "inreq_allowed", &mode)) {
+
+		if (0 == pl_strcasecmp(&mode, "no")) {
+			prm->inreq_mode = INREQ_MODE_OFF;
+		}
+		else if (0 == pl_strcasecmp(&mode, "yes")) {
+			prm->inreq_mode = INREQ_MODE_ON;
+		}
+		else {
+			warning("account: inreq_allowed mode unknown (%r)\n",
+				&mode);
+		}
+	}
+}
+
+
 static int csl_parse(struct pl *pl, char *str, size_t sz)
 {
 	struct pl ws = PL_INIT, val, ws2 = PL_INIT, cma = PL_INIT;
@@ -284,7 +405,7 @@ static int audio_codecs_decode(struct account *acc, const struct pl *prm)
 			/* NOTE: static list with references to aucodec */
 			list_append(&acc->aucodecl, &acc->acv[i++], ac);
 
-			if (i >= ARRAY_SIZE(acc->acv))
+			if (i >= RE_ARRAY_SIZE(acc->acv))
 				break;
 		}
 	}
@@ -308,6 +429,7 @@ static int video_codecs_decode(struct account *acc, const struct pl *prm)
 		char cname[64];
 		unsigned i = 0;
 
+		acc->videoen = false;
 		if (msg_param_decode(prm, "video_codecs", &vcs))
 			return 0;
 
@@ -324,7 +446,8 @@ static int video_codecs_decode(struct account *acc, const struct pl *prm)
 				list_append(&acc->vidcodecl, &acc->vcv[i++],
 						vc);
 
-				if (i >= ARRAY_SIZE(acc->vcv))
+				acc->videoen = true;
+				if (i >= RE_ARRAY_SIZE(acc->vcv))
 					return 0;
 			}
 		}
@@ -334,11 +457,32 @@ static int video_codecs_decode(struct account *acc, const struct pl *prm)
 }
 
 
+static void uasauth_decode(struct account *acc, const struct pl *prm)
+{
+	struct pl val;
+
+	if (!acc || !prm)
+		return;
+
+	if (msg_param_decode(prm, "uas_user", &val))
+		return;
+
+	(void)pl_strdup(&acc->uas_user, &val);
+
+	if (msg_param_decode(prm, "uas_pass", &val))
+		return;
+
+	(void)pl_strdup(&acc->uas_pass, &val);
+}
+
+
 static int sip_params_decode(struct account *acc, const struct sip_addr *aor)
 {
 	struct pl auth_user, tmp;
 	size_t i;
+	uint32_t u32;
 	int err = 0;
+	char *value;
 
 	if (!acc || !aor)
 		return EINVAL;
@@ -356,10 +500,18 @@ static int sip_params_decode(struct account *acc, const struct sip_addr *aor)
 	err |= param_u32(&acc->fbregint, &aor->params, "fbregint");
 	acc->pubint = 0;
 	err |= param_u32(&acc->pubint, &aor->params, "pubint");
+	u32  = 0;
+	err |= param_u32(&u32, &aor->params, "tcpsrcport");
+	if (u32) {
+		if (u32 <= 65535)
+			acc->tcpsrcport = u32;
+		else
+			warning("account: invalid tcpsrcport\n");
+	}
 
 	err |= param_dstr(&acc->regq, &aor->params, "regq");
 
-	for (i=0; i<ARRAY_SIZE(acc->outboundv); i++) {
+	for (i=0; i<RE_ARRAY_SIZE(acc->outboundv); i++) {
 
 		char expr[16] = "outbound";
 
@@ -375,7 +527,10 @@ static int sip_params_decode(struct account *acc, const struct sip_addr *aor)
 				  "outbound");
 	}
 
-	err |= param_dstr(&acc->sipnat, &aor->params, "sipnat");
+	value = NULL;
+	err |= param_dstr(&value, &aor->params, "sipnat");
+	(void)account_set_sipnat(acc, value);
+	mem_deref(value);
 
 	if (0 == msg_param_decode(&aor->params, "auth_user", &auth_user))
 		err |= pl_strdup(&acc->auth_user, &auth_user);
@@ -393,6 +548,11 @@ static int sip_params_decode(struct account *acc, const struct sip_addr *aor)
 	else
 		acc->refer = pl_strcasecmp(&tmp, "no") != 0;
 
+	if (0 != msg_param_decode(&aor->params, "sip_autoredirect", &tmp))
+		acc->autoredirect = false;
+	else
+		acc->autoredirect = pl_strcasecmp(&tmp, "yes") == 0;
+
 	return err;
 }
 
@@ -428,6 +588,8 @@ int account_alloc(struct account **accp, const char *sipaddr)
 	if (!acc)
 		return ENOMEM;
 
+	acc->sipansbeep = SIPANSBEEP_ON;
+	acc->videoen = true;
 	err = str_dup(&acc->buf, sipaddr);
 	if (err)
 		goto out;
@@ -449,15 +611,20 @@ int account_alloc(struct account **accp, const char *sipaddr)
 	/* Decode parameters */
 	acc->ptime = 20;
 	err |= sip_params_decode(acc, &acc->laddr);
+	       rel100_decode(acc, &acc->laddr.params);
 	       answermode_decode(acc, &acc->laddr.params);
-	       autoanswer_allow_decode(acc, &acc->laddr.params);
-	       dtmfmode_decode(acc,&acc->laddr.params);
+	       autoanswer_decode(acc, &acc->laddr.params);
+	       dtmfmode_decode(acc, &acc->laddr.params);
+	       uasauth_decode(acc, &acc->laddr.params);
+	       inreq_mode_decode(acc, &acc->laddr.params);
 	err |= audio_codecs_decode(acc, &acc->laddr.params);
 	err |= video_codecs_decode(acc, &acc->laddr.params);
 	err |= media_decode(acc, &acc->laddr.params);
+	err |= param_bool(&acc->catchall, &acc->laddr.params, "catchall");
 	if (err)
 		goto out;
 
+	param_u32(&acc->autelev_pt, &acc->laddr.params, "autelev_pt");
 	err  = decode_pair(&acc->ausrc_mod, &acc->ausrc_dev,
 			   &acc->laddr.params, "audio_source");
 	err |= decode_pair(&acc->auplay_mod, &acc->auplay_dev,
@@ -494,6 +661,7 @@ int account_alloc(struct account **accp, const char *sipaddr)
 		}
 	}
 
+	err |= cert_decode(acc, &acc->laddr.params);
 	err |= extra_decode(acc, &acc->laddr.params);
 
  out:
@@ -561,7 +729,7 @@ int account_set_auth_pass(struct account *acc, const char *pass)
  */
 int account_set_outbound(struct account *acc, const char *ob, unsigned ix)
 {
-	if (!acc || ix >= ARRAY_SIZE(acc->outboundv))
+	if (!acc || ix >= RE_ARRAY_SIZE(acc->outboundv))
 		return EINVAL;
 
 	acc->outboundv[ix] = mem_deref(acc->outboundv[ix]);
@@ -586,10 +754,18 @@ int account_set_sipnat(struct account *acc, const char *sipnat)
 	if (!acc)
 		return EINVAL;
 
-	acc->sipnat = mem_deref(acc->sipnat);
-
 	if (sipnat)
-		return str_dup(&acc->sipnat, sipnat);
+		if (0 == str_casecmp(sipnat, "outbound")) {
+			acc->sipnat = mem_deref(acc->sipnat);
+			return str_dup(&acc->sipnat, sipnat);
+		}
+		else {
+			warning("account: unknown sipnat value: '%s'\n",
+				sipnat);
+			return EINVAL;
+		}
+	else
+		acc->sipnat = mem_deref(acc->sipnat);
 
 	return 0;
 }
@@ -879,61 +1055,78 @@ int account_set_display_name(struct account *acc, const char *dname)
 
 
 /**
- * Sets MWI on (value "yes") or off (value "no")
+ * Sets MWI on (value true) or off (value false)
  *
  * @param acc      User-Agent account
- * @param value    "yes" or "no"
+ * @param value    true or false
  *
  * @return 0 if success, otherwise errorcode
  */
-int account_set_mwi(struct account *acc, const char *value)
+int account_set_mwi(struct account *acc, bool value)
 {
 	if (!acc)
 		return EINVAL;
 
-	if (0 == str_casecmp(value, "yes"))
-		acc->mwi = true;
-	else
-		if (0 == str_casecmp(value, "no"))
-			acc->mwi = false;
-		else {
-			warning("account: unknown mwi value: %r\n",
-				value);
-			return EINVAL;
-		}
+	acc->mwi = value;
 
 	return 0;
 }
 
 
 /**
- * Sets call transfer on (value "yes") or off (value "no")
+ * Sets call transfer on (value true) or off (value false)
  *
  * @param acc      User-Agent account
- * @param value    "yes" or "no"
+ * @param value    true or false
  *
  * @return 0 if success, otherwise errorcode
  */
-int account_set_call_transfer(struct account *acc, const char *value)
+int account_set_call_transfer(struct account *acc, bool value)
 {
 	if (!acc)
 		return EINVAL;
 
-	if (0 == str_casecmp(value, "yes"))
-		acc->refer = true;
-	else
-		if (0 == str_casecmp(value, "no"))
-			acc->refer = false;
-		else {
-			warning("account: unknown call transfer: %r\n",
-				value);
-			return EINVAL;
-		}
+	acc->refer = value;
 
 	return 0;
 }
 
 
+/**
+ * Sets rtcp_mux on (value true) or off (value false)
+ *
+ * @param acc      User-Agent account
+ * @param value    true or false
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int account_set_rtcp_mux(struct account *acc, bool value)
+{
+	if (!acc)
+		return EINVAL;
+
+	acc->rtcp_mux = value;
+
+	return 0;
+}
+
+
+/**
+ * Sets catchall flag for the User-Agent account. A catch all account catches
+ * all inbound SIP requests
+ *
+ * @param acc      User-Agent account
+ * @param value    true or false
+ */
+void account_set_catchall(struct account *acc, bool value)
+{
+	if (!acc)
+		return;
+
+	acc->catchall = value;
+}
+
+
 /**
  * Authenticate a User-Agent (UA)
  *
@@ -984,10 +1177,13 @@ struct list *account_aucodecl(const struct account *acc)
  *
  * @param acc User-Agent account
  *
- * @return List of video codecs (struct vidcodec)
+ * @return List of video codecs (struct vidcodec), NULL if video is disabled
  */
 struct list *account_vidcodecl(const struct account *acc)
 {
+	if (acc && !acc->videoen)
+		return NULL;
+
 	return (acc && !list_isempty(&acc->vidcodecl))
 		? (struct list *)&acc->vidcodecl : baresip_vidcodecl();
 }
@@ -1098,7 +1294,8 @@ int account_set_answermode(struct account *acc, enum answermode mode)
 		return EINVAL;
 
 	if ((mode != ANSWERMODE_MANUAL) && (mode != ANSWERMODE_EARLY) &&
-	    (mode != ANSWERMODE_AUTO)) {
+	    (mode != ANSWERMODE_AUTO) && (mode != ANSWERMODE_EARLY_VIDEO) &&
+	    (mode != ANSWERMODE_EARLY_AUDIO)) {
 		warning("account: invalid answermode : `%d'\n", mode);
 		return EINVAL;
 	}
@@ -1109,6 +1306,44 @@ int account_set_answermode(struct account *acc, enum answermode mode)
 }
 
 
+/**
+ * Get the 100rel mode of an account
+ *
+ * @param acc User-Agent account
+ *
+ * @return 100rel mode
+ */
+enum rel100_mode account_rel100_mode(const struct account *acc)
+{
+	return acc ? acc->rel100_mode : REL100_ENABLED;
+}
+
+
+/**
+ * Set the 100rel mode of an account
+ *
+ * @param acc  User-Agent account
+ * @param mode 100rel mode
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int account_set_rel100_mode(struct account *acc, enum rel100_mode mode)
+{
+	if (!acc)
+		return EINVAL;
+
+	if ((mode != REL100_DISABLED) && (mode != REL100_ENABLED) &&
+	    (mode != REL100_REQUIRED)) {
+		warning("account: invalid 100rel mode : `%d'\n", mode);
+		return EINVAL;
+	}
+
+	acc->rel100_mode = mode;
+
+	return 0;
+}
+
+
 /**
  * Get the dtmfmode of an account
  *
@@ -1135,7 +1370,9 @@ int account_set_dtmfmode(struct account *acc, enum dtmfmode mode)
 	if (!acc)
 		return EINVAL;
 
-	if ((mode != DTMFMODE_RTP_EVENT) && (mode != DTMFMODE_SIP_INFO)) {
+	if ((mode != DTMFMODE_RTP_EVENT) &&
+	    (mode != DTMFMODE_SIP_INFO) &&
+	    (mode != DTMFMODE_AUTO)) {
 		warning("account: invalid dtmfmode : `%d'\n", mode);
 		return EINVAL;
 	}
@@ -1208,7 +1445,7 @@ const char *account_auth_pass(const struct account *acc)
  */
 const char *account_outbound(const struct account *acc, unsigned ix)
 {
-	if (!acc || ix >= ARRAY_SIZE(acc->outboundv))
+	if (!acc || ix >= RE_ARRAY_SIZE(acc->outboundv))
 		return NULL;
 
 	return acc->outboundv[ix];
@@ -1315,6 +1552,21 @@ uint16_t account_stun_port(const struct account *acc)
 }
 
 
+int32_t account_answerdelay(const struct account *acc)
+{
+	return acc ? acc->adelay : 0;
+}
+
+
+void account_set_answerdelay(struct account *acc, int32_t adelay)
+{
+	if (!acc)
+		return;
+
+	acc->adelay = adelay;
+}
+
+
 /**
  * Returns if SIP auto answer is allowed for the account
  *
@@ -1327,13 +1579,115 @@ bool account_sip_autoanswer(const struct account *acc)
 }
 
 
+void account_set_sip_autoanswer(struct account *acc, bool allow)
+{
+	if (!acc)
+		return;
+
+	acc->sipans = allow;
+}
+
+
+/**
+ * Returns if SIP autoredirect on 3xx response is allowed for the account
+ *
+ * @param acc User-Agent account
+ * @return true if allowed, otherwise false
+ */
+bool account_sip_autoredirect(const struct account *acc)
+{
+	return acc ? acc->autoredirect : false;
+}
+
+
+void account_set_sip_autoredirect(struct account *acc, bool allow)
+{
+	if (!acc)
+		return;
+
+	acc->autoredirect = allow;
+}
+
+
+/**
+ * Returns the beep mode for a SIP auto answer call
+ *
+ * - SIPANSBEEP_ON    ... The beep is played before the call is answered
+ *                         automatically. The locally configured audio file can
+ *                         be overwritten with the Alert-Info header URL.
+ *                         This is the default value.
+ *
+ * - SIPANSBEEP_OFF   ... No beep is played.
+ *
+ * - SIPANSBEEP_LOCAL ... The local configured beep tone is played.
+ *
+ * @param acc User-Agent account
+ * @return Beep mode
+ */
+enum sipansbeep account_sipansbeep(const struct account *acc)
+{
+	return acc ? acc->sipansbeep : SIPANSBEEP_ON;
+}
+
+
+void account_set_sipansbeep(struct account *acc, enum sipansbeep beep)
+{
+	if (!acc)
+		return;
+
+	acc->sipansbeep = beep;
+}
+
+
+/**
+ * Sets the audio payload type for telephone-events
+ *
+ * @param acc User-Agent account
+ * @param pt  Payload type
+ */
+void account_set_autelev_pt(struct account *acc, uint32_t pt)
+{
+	if (!acc)
+		return;
+
+	acc->autelev_pt = pt;
+}
+
+
+/**
+ * Returns the audio payload type for telephone-events
+ *
+ * @param acc User-Agent account
+ *
+ * @return Telephone-event payload type
+ */
+uint32_t account_autelev_pt(struct account *acc)
+{
+	return acc ? acc->autelev_pt : 0;
+}
+
+
 static const char *answermode_str(enum answermode mode)
 {
 	switch (mode) {
 
-	case ANSWERMODE_MANUAL: return "manual";
-	case ANSWERMODE_EARLY:  return "early";
-	case ANSWERMODE_AUTO:   return "auto";
+	case ANSWERMODE_MANUAL:       return "manual";
+	case ANSWERMODE_EARLY:        return "early";
+	case ANSWERMODE_AUTO:         return "auto";
+	case ANSWERMODE_EARLY_AUDIO:  return "early-audio";
+	case ANSWERMODE_EARLY_VIDEO:  return "early-video";
+	default: return "???";
+	}
+}
+
+
+static const char *rel100_mode_str(enum rel100_mode mode)
+{
+	switch (mode) {
+
+	case REL100_ENABLED:    return "yes";
+	case REL100_DISABLED:   return "no";
+	case REL100_REQUIRED:   return "required";
 	default: return "???";
 	}
 }
@@ -1345,6 +1699,30 @@ static const char *dtmfmode_str(enum dtmfmode mode)
 
 	case DTMFMODE_RTP_EVENT: return "rtpevent";
 	case DTMFMODE_SIP_INFO:  return "info";
+	case DTMFMODE_AUTO: 	 return "auto";
+	default: return "???";
+	}
+}
+
+
+static const char *sipansbeep_str(enum sipansbeep beep)
+{
+	switch (beep) {
+
+	case SIPANSBEEP_OFF:   return "off";
+	case SIPANSBEEP_ON:    return "on";
+	case SIPANSBEEP_LOCAL: return "local";
+	default: return "???";
+	}
+}
+
+
+static const char *inreq_mode_str(enum inreq_mode mode)
+{
+	switch (mode) {
+
+	case INREQ_MODE_OFF:      return "no";
+	case INREQ_MODE_ON:       return "yes";
 	default: return "???";
 	}
 }
@@ -1381,14 +1759,14 @@ const char *account_medianat(const struct account *acc)
  *
  * @param acc User-Agent account
  *
- * @return "yes" or "no"
+ * @return true or false
  */
-const char *account_mwi(const struct account *acc)
+bool account_mwi(const struct account *acc)
 {
 	if (!acc)
-		return "no";
+		return false;
 
-	return acc->mwi ? "yes" : "no";
+	return acc->mwi;
 }
 
 
@@ -1397,14 +1775,30 @@ const char *account_mwi(const struct account *acc)
  *
  * @param acc User-Agent account
  *
- * @return "yes" or "no"
+ * @return true or false
  */
-const char *account_call_transfer(const struct account *acc)
+bool account_call_transfer(const struct account *acc)
 {
 	if (!acc)
-		return "no";
+		return false;
 
-	return acc->refer ? "yes" : "no";
+	return acc->refer;
+}
+
+
+/**
+ * Get rtcp_mux capability of an account
+ *
+ * @param acc User-Agent account
+ *
+ * @return true or false
+ */
+bool account_rtcp_mux(const struct account *acc)
+{
+	if (!acc)
+		return false;
+
+	return acc->rtcp_mux;
 }
 
 
@@ -1432,63 +1826,104 @@ const char *account_extra(const struct account *acc)
  */
 int account_uri_complete(const struct account *acc, struct mbuf *buf,
 			 const char *uri)
+{
+	char *str;
+	struct pl pl;
+	int err;
+
+	pl_set_str(&pl, uri);
+	err = account_uri_complete_strdup(acc, &str, &pl);
+	if (err)
+		return err;
+
+	err = mbuf_write_str(buf, str);
+	mem_deref(str);
+
+	return err;
+}
+
+
+/**
+ * Auto complete a SIP uri, add scheme and domain if missing
+ *
+ * @param acc  User-Agent account
+ * @param strp Pointer to destination string; allocated and set
+ * @param uri  Input SIP uri pointer-length string
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int account_uri_complete_strdup(const struct account *acc, char **strp,
+				const struct pl *uri)
 {
 	struct sa sa_addr;
-	size_t len;
 	bool uri_is_ip;
 	char *uridup;
 	char *host;
-	char *param;
+	char *c;
+	bool complete = false;
+	struct mbuf *mb;
+	struct pl trimmed;
 	int err = 0;
 
-	if (!buf || !uri)
+	if (!strp || !pl_isset(uri))
 		return EINVAL;
 
 	/* Skip initial whitespace */
-	while (isspace(*uri))
-		++uri;
+	trimmed = *uri;
+	err = pl_ltrim(&trimmed);
+	if (err)
+		return err;
 
-	len = str_len(uri);
+	mb = mbuf_alloc(64);
+	if (!mb)
+		return ENOMEM;
 
 	/* Append sip: scheme if missing */
-	if (0 != re_regex(uri, len, "sip:"))
-		err |= mbuf_printf(buf, "sip:");
-
-	err |= mbuf_write_str(buf, uri);
+	if (re_regex(trimmed.p, trimmed.l, "sip:"))
+		err |= mbuf_printf(mb, "sip:");
+	else
+		complete = true;
 
+	err |= mbuf_write_pl(mb, &trimmed);
 	if (!acc || err)
-		return err;
+		goto out;
+
+	if (complete)
+		goto out;
 
 	/* Append domain if missing and uri is not IP address */
 
 	/* check if uri is valid IP address */
-	err = str_dup(&uridup, uri);
+	err = pl_strdup(&uridup, &trimmed);
 	if (err)
-		return err;
+		goto out;
 
 	if (!strncmp(uridup, "sip:", 4))
 		host = uridup + 4;
 	else
 		host = uridup;
 
-	param = strchr(host, ';');
-	if (param)
-		*param = 0;
+	c = strchr(host, ';');
+	if (c)
+		*c = 0;
 
 	uri_is_ip =
 		!sa_decode(&sa_addr, host, strlen(host)) ||
 		!sa_set_str(&sa_addr, host, 0);
+	if (!uri_is_ip && host[0] == '[' && (c = strchr(host, ']'))) {
+		*c = 0;
+		uri_is_ip = !sa_set_str(&sa_addr, host+1, 0);
+	}
+
 	mem_deref(uridup);
 
-	if (0 != re_regex(uri, len, "[^@]+@[^]+", NULL, NULL) &&
-		1 != uri_is_ip) {
-#if HAVE_INET6
+	if (!uri_is_ip &&
+	    0 != re_regex(trimmed.p, trimmed.l, "[^@]+@[^]+", NULL, NULL)) {
 		if (AF_INET6 == acc->luri.af)
-			err |= mbuf_printf(buf, "@[%r]",
+			err |= mbuf_printf(mb, "@[%r]",
 					   &acc->luri.host);
 		else
-#endif
-			err |= mbuf_printf(buf, "@%r",
+			err |= mbuf_printf(mb, "@%r",
 					   &acc->luri.host);
 
 		/* Also append port if specified and not 5060 */
@@ -1499,11 +1934,18 @@ int account_uri_complete(const struct account *acc, struct mbuf *buf,
 			break;
 
 		default:
-			err |= mbuf_printf(buf, ":%u", acc->luri.port);
+			err |= mbuf_printf(mb, ":%u", acc->luri.port);
 			break;
 		}
 	}
 
+out:
+	if (!err) {
+		mbuf_set_pos(mb, 0);
+		err = mbuf_strdup(mb, strp, mbuf_get_left(mb));
+	}
+
+	mem_deref(mb);
 	return err;
 }
 
@@ -1532,10 +1974,16 @@ int account_debug(struct re_printf *pf, const struct account *acc)
 			  uri_encode, &acc->luri);
 	err |= re_hprintf(pf, " aor:          %s\n", acc->aor);
 	err |= re_hprintf(pf, " dispname:     %s\n", acc->dispname);
+	err |= re_hprintf(pf, " 100rel:       %s\n",
+			  rel100_mode_str(acc->rel100_mode));
 	err |= re_hprintf(pf, " answermode:   %s\n",
 			  answermode_str(acc->answermode));
+	err |= re_hprintf(pf, " autoredirect:   %s\n",
+			  acc->autoredirect ? "yes" : "no");
 	err |= re_hprintf(pf, " sipans:       %s\n",
 			  acc->sipans ? "yes" : "no");
+	err |= re_hprintf(pf, " sipansbeep:   %s\n",
+			  sipansbeep_str(acc->sipansbeep));
 	err |= re_hprintf(pf, " dtmfmode:     %s\n",
 			  dtmfmode_str(acc->dtmfmode));
 	if (!list_isempty(&acc->aucodecl)) {
@@ -1547,27 +1995,35 @@ int account_debug(struct re_printf *pf, const struct account *acc)
 		}
 		err |= re_hprintf(pf, "\n");
 	}
+	err |= re_hprintf(pf, " autelev_pt:   %u\n", acc->autelev_pt);
 	err |= re_hprintf(pf, " auth_user:    %s\n", acc->auth_user);
 	err |= re_hprintf(pf, " mediaenc:     %s\n",
 			  acc->mencid ? acc->mencid : "none");
 	err |= re_hprintf(pf, " medianat:     %s\n",
 			  acc->mnatid ? acc->mnatid : "none");
-	for (i=0; i<ARRAY_SIZE(acc->outboundv); i++) {
+	err |= re_hprintf(pf, " natpinhole:   %s\n",
+			  acc->pinhole ? "yes" : "no");
+	for (i=0; i<RE_ARRAY_SIZE(acc->outboundv); i++) {
 		if (acc->outboundv[i]) {
-			err |= re_hprintf(pf, " outbound%d:    %s\n",
+			err |= re_hprintf(pf, " outbound%zu:    %s\n",
 					  i+1, acc->outboundv[i]);
 		}
 	}
-	err |= re_hprintf(pf, " mwi:          %s\n", account_mwi(acc));
+	err |= re_hprintf(pf, " mwi:          %s\n",
+			  account_mwi(acc) ? "yes" : "no");
 	err |= re_hprintf(pf, " ptime:        %u\n", acc->ptime);
 	err |= re_hprintf(pf, " regint:       %u\n", acc->regint);
 	err |= re_hprintf(pf, " prio:         %u\n", acc->prio);
 	err |= re_hprintf(pf, " pubint:       %u\n", acc->pubint);
 	err |= re_hprintf(pf, " regq:         %s\n", acc->regq);
+	err |= re_hprintf(pf, " inreq_allowed:%s\n",
+			  inreq_mode_str(acc->inreq_mode));
 	err |= re_hprintf(pf, " sipnat:       %s\n", acc->sipnat);
 	err |= re_hprintf(pf, " stunuser:     %s\n", acc->stun_user);
 	err |= re_hprintf(pf, " stunserver:   %H\n",
 			  stunuri_print, acc->stun_host);
+	err |= re_hprintf(pf, " rtcp_mux:     %s\n",
+			  acc->rtcp_mux ? "yes" : "no");
 
 	if (!list_isempty(&acc->vidcodecl)) {
 		err |= re_hprintf(pf, " video_codecs:");
@@ -1578,7 +2034,9 @@ int account_debug(struct re_printf *pf, const struct account *acc)
 		err |= re_hprintf(pf, "\n");
 	}
 	err |= re_hprintf(pf, " call_transfer:%s\n",
-			  account_call_transfer(acc));
+			  account_call_transfer(acc) ? "yes" : "no");
+	err |= re_hprintf(pf, " catchall:%s\n", acc->catchall ? "yes" : "no");
+	err |= re_hprintf(pf, " cert:         %s\n", acc->cert);
 	err |= re_hprintf(pf, " extra:        %s\n",
 			  acc->extra ? acc->extra : "none");
 
@@ -1620,7 +2078,7 @@ int account_json_api(struct odict *od, struct odict *odcfg,
 	}
 
 	err |= odict_alloc(&obn, 8);
-	for (i=0; i<ARRAY_SIZE(acc->outboundv); i++) {
+	for (i=0; i<RE_ARRAY_SIZE(acc->outboundv); i++) {
 		if (acc->outboundv[i]) {
 			err |= odict_entry_add(obn, "outbound", ODICT_STRING,
 					acc->outboundv[i]);
@@ -1637,8 +2095,12 @@ int account_json_api(struct odict *od, struct odict *odcfg,
 				acc->stun_user);
 	}
 
+	err |= odict_entry_add(odcfg, "rel100_mode", ODICT_STRING,
+			rel100_mode_str(acc->rel100_mode));
 	err |= odict_entry_add(odcfg, "answer_mode", ODICT_STRING,
 			answermode_str(acc->answermode));
+	err |= odict_entry_add(odcfg, "inreq_allowed", ODICT_STRING,
+			inreq_mode_str(acc->inreq_mode));
 	err |= odict_entry_add(odcfg, "call_transfer", ODICT_BOOL, acc->refer);
 
 	err |= odict_entry_add(odcfg, "packet_time", ODICT_INT,
@@ -1647,3 +2109,67 @@ int account_json_api(struct odict *od, struct odict *odcfg,
 	mem_deref(obn);
 	return err;
 }
+
+
+const char* account_uas_user(const struct account *acc)
+{
+	if (!acc)
+		return NULL;
+
+	return acc->uas_user;
+}
+
+
+const char* account_uas_pass(const struct account *acc)
+{
+	if (!acc)
+		return NULL;
+
+	return acc->uas_pass;
+}
+
+
+bool account_uas_isset(const struct account *acc)
+{
+	if (!acc)
+		return false;
+
+	return acc->uas_user || acc->uas_pass;
+}
+
+
+/**
+ * Get the incoming out-of-dialog request mode of an account
+ *
+ * @param acc User-Agent account
+ *
+ * @return inreq_mode
+ */
+enum inreq_mode account_inreq_mode(const struct account *acc)
+{
+	return acc ? acc->inreq_mode : INREQ_MODE_ON;
+}
+
+
+/**
+ * Set the incoming out-of-dialog request mode of an account
+ *
+ * @param acc  User-Agent account
+ * @param mode Incoming request mode
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int account_set_inreq_mode(struct account *acc, enum inreq_mode mode)
+{
+	if (!acc)
+		return EINVAL;
+
+	if ((mode != INREQ_MODE_OFF) && (mode != INREQ_MODE_ON)) {
+		warning("account: invalid inreq_allowed : '%d'\n", mode);
+		return EINVAL;
+	}
+
+	acc->inreq_mode = mode;
+
+	return 0;
+}
