@@ -17,7 +17,7 @@ enum {PTIME = 40};
 struct play {
 	struct le le;
 	struct play **playp;
-	struct lock *lock;
+	mtx_t lock;
 	struct mbuf *mb;
 	struct auplay_st *auplay;
 	char *mod;
@@ -32,8 +32,6 @@ struct play {
 	const struct ausrc *ausrc;
 	struct ausrc_st *ausrc_st;
 	struct ausrc_prm sprm;
-	size_t minsz;
-	size_t maxsz;
 	struct aubuf *aubuf;
 	play_finish_h *fh;
 	void *arg;
@@ -67,8 +65,9 @@ static void tmr_stop(void *arg)
 static void tmr_polling(void *arg)
 {
 	struct play *play = arg;
+	int err = 0;
 
-	lock_write_get(play->lock);
+	mtx_lock(&play->lock);
 
 	tmr_start(&play->tmr, PTIME, tmr_polling, play);
 
@@ -77,13 +76,21 @@ static void tmr_polling(void *arg)
 			tmr_start(&play->tmr, 1, tmr_stop, arg);
 	}
 	else if (play->aubuf && !play->auplay) {
-		if (aubuf_cur_size(play->aubuf))
-			start_auplay(play);
+		err = start_auplay(play);
 
 		tmr_start(&play->tmr, 4, tmr_polling, play);
 	}
 
-	lock_rel(play->lock);
+	if (play->ausrc && play->trep && play->trep <= tmr_jiffies()) {
+		play->trep = 0;
+		aubuf_flush(play->aubuf);
+		err = start_ausrc(play);
+	}
+
+	mtx_unlock(&play->lock);
+
+	if (err)
+		tmr_start(&play->tmr, 1, tmr_stop, arg);
 }
 
 
@@ -122,7 +129,7 @@ static void write_handler(struct auframe *af, void *arg)
 	size_t left;
 	size_t count;
 
-	lock_write_get(play->lock);
+	mtx_lock(&play->lock);
 
 	if (play->eof)
 		goto silence;
@@ -148,7 +155,7 @@ static void write_handler(struct auframe *af, void *arg)
 	if (play->eof)
 		memset((uint8_t *)af->sampv + pos, 0, sz - pos);
 
-	lock_rel(play->lock);
+	mtx_unlock(&play->lock);
 }
 
 
@@ -159,16 +166,16 @@ static void destructor(void *arg)
 	list_unlink(&play->le);
 	tmr_cancel(&play->tmr);
 
-	lock_write_get(play->lock);
+	mtx_lock(&play->lock);
 	play->eof = true;
-	lock_rel(play->lock);
+	mtx_unlock(&play->lock);
 
 	mem_deref(play->ausrc_st);
 	mem_deref(play->auplay);
 	mem_deref(play->mod);
 	mem_deref(play->dev);
 	mem_deref(play->mb);
-	mem_deref(play->lock);
+	mtx_destroy(&play->lock);
 	mem_deref(play->aubuf);
 	mem_deref(play->filename);
 
@@ -282,9 +289,11 @@ int play_tone(struct play **playp, struct player *player,
 	play->repeat = repeat ? repeat : 1;
 	play->mb     = mem_ref(tone);
 
-	err = lock_alloc(&play->lock);
-	if (err)
+	err = mtx_init(&play->lock, mtx_plain) != thrd_success;
+	if (err) {
+		err = ENOMEM;
 		goto out;
+	}
 
 	wprm.ch         = ch;
 	wprm.srate      = srate;
@@ -316,14 +325,13 @@ int play_tone(struct play **playp, struct player *player,
 static void ausrc_read_handler(struct auframe *af, void *arg)
 {
 	struct play *play = arg;
-	size_t fsize    = auframe_size(af);
 	int err = 0;
 
 	if (play->eof)
 		return;
 
 
-	err = aubuf_write(play->aubuf, af->sampv, fsize);
+	err = aubuf_write_auframe(play->aubuf, af);
 	if (err)
 		warning("play: aubuf_write: %m \n", err);
 }
@@ -332,20 +340,19 @@ static void ausrc_read_handler(struct auframe *af, void *arg)
 static void aubuf_write_handler(struct auframe *af, void *arg)
 {
 	struct play *play = arg;
-	size_t sz = af->sampc * aufmt_sample_size(play->sprm.fmt);
+	size_t sz = auframe_size(af);
 	size_t left = aubuf_cur_size(play->aubuf);
 
-	aubuf_read(play->aubuf, af->sampv, sz);
+	aubuf_read_auframe(play->aubuf, af);
 
-	lock_write_get(play->lock);
-	if (!play->trep && !play->ausrc_st && left < sz) {
-		check_restart(play);
-	}
-	else if (play->trep && check_restart(play)) {
-		start_ausrc(play);
+	mtx_lock(&play->lock);
+	if (!play->trep && !play->ausrc_st) {
+		bool filling = left == aubuf_cur_size(play->aubuf);
+		if (left < sz || filling)
+			check_restart(play);
 	}
 
-	lock_rel(play->lock);
+	mtx_unlock(&play->lock);
 }
 
 
@@ -355,9 +362,9 @@ static void ausrc_error_handler(int err, const char *str, void *arg)
 	(void)str;
 
 	if (err == 0) {
-		lock_write_get(play->lock);
+		mtx_lock(&play->lock);
 		play->ausrc_st = mem_deref(play->ausrc_st);
-		lock_rel(play->lock);
+		mtx_unlock(&play->lock);
 	}
 }
 
@@ -367,9 +374,14 @@ static int start_ausrc(struct play *play)
 	int err;
 	const struct ausrc *ausrc = play->ausrc;
 
-	err = ausrc->alloch(&play->ausrc_st, ausrc, NULL, &play->sprm,
+	if (!ausrc)
+		return EINVAL;
+
+	err = ausrc->alloch(&play->ausrc_st, ausrc, &play->sprm,
 			play->filename,
 			ausrc_read_handler, ausrc_error_handler, play);
+	if (err)
+		warning("play: could not start ausrc (%m)\n", err);
 
 	return err;
 }
@@ -388,6 +400,9 @@ static int start_auplay(struct play *play)
 	err = auplay_alloc(&play->auplay, baresip_auplayl(),
 			   play->mod, &wprm,
 			   play->dev, aubuf_write_handler, play);
+	if (err)
+		warning("play: could not start auplay (%m)\n", err);
+
 	return err;
 }
 
@@ -403,6 +418,8 @@ static int play_file_ausrc(struct play **playp,
 	struct play *play;
 	uint32_t srate = 0;
 	uint32_t channels = 0;
+	size_t minsz;
+	size_t maxsz;
 
 	conf_get_u32(conf_cur(), "file_srate", &srate);
 	conf_get_u32(conf_cur(), "file_channels", &channels);
@@ -417,9 +434,11 @@ static int play_file_ausrc(struct play **playp,
 	if (!channels)
 		channels = 1;
 
-	err = lock_alloc(&play->lock);
-	if (err)
+	err = mtx_init(&play->lock, mtx_plain) != thrd_success;
+	if (err) {
+		err = ENOMEM;
 		goto out;
+	}
 
 	str_dup(&play->mod, play_mod);
 	str_dup(&play->dev, play_dev);
@@ -434,12 +453,13 @@ static int play_file_ausrc(struct play **playp,
 	str_dup(&play->filename, filename);
 
 	sampsz = aufmt_sample_size(sprm.fmt);
-	play->maxsz = (24 * sampsz * srate * channels * PTIME) / 1000;
-	play->minsz = ( 3 * sampsz * srate * channels * PTIME) / 1000;
-	err = aubuf_alloc(&play->aubuf, play->minsz, play->maxsz);
+	minsz =  3 * sampsz * srate * channels * PTIME / 1000;
+	maxsz = 24 * sampsz * srate * channels * PTIME / 1000;
+	err = aubuf_alloc(&play->aubuf, minsz, maxsz);
 	if (err)
 		goto out;
 
+	aubuf_set_live(play->aubuf, false);
 	play->ausrc = ausrc;
 	err = start_ausrc(play);
 	if (err)
@@ -527,11 +547,19 @@ int play_file(struct play **playp, struct player *player,
 	str_ncpy(file, filename, sizeof(file));
 	parse_play_settings(file, &repeat, &delay);
 
-	if (re_snprintf(path, sizeof(path), "%s/%s",
+	/* absolute path? */
+	if (file[0] == '/' ||
+	    !re_regex(file, strlen(file), "https://") ||
+	    !re_regex(file, strlen(file), "http://") ||
+	    !re_regex(file, strlen(file), "file://")) {
+		if (re_snprintf(path, sizeof(path), "%s",
+				file) < 0)
+			return ENOMEM;
+	}
+	else if (re_snprintf(path, sizeof(path), "%s/%s",
 			player->play_path, file) < 0)
 		return ENOMEM;
 
-
 	if (!conf_get_str(conf_cur(), "file_ausrc", srcn, sizeof(srcn))) {
 		ausrc = ausrc_find(baresip_ausrcl(), srcn);
 		if (ausrc) {
@@ -564,7 +592,7 @@ int play_file(struct play **playp, struct player *player,
 	if (err) {
 		mem_deref(play);
 	}
-	else if (playp) {
+	else if (play && playp) {
 		play->playp = playp;
 		*playp = play;
 	}
