@@ -20,6 +20,8 @@ struct reg {
 	uint16_t scode;              /**< Registration status code           */
 	char *srv;                   /**< SIP Server id                      */
 	int af;                      /**< Cached address family for SIP conn */
+
+	struct list custom_hdrs;     /**< List of custom headers if any      */
 };
 
 
@@ -30,6 +32,8 @@ static void destructor(void *arg)
 	list_unlink(&reg->le);
 	mem_deref(reg->sipreg);
 	mem_deref(reg->srv);
+
+	list_flush(&reg->custom_hdrs);
 }
 
 
@@ -191,6 +195,33 @@ int reg_add(struct list *lst, struct ua *ua, int regid)
 }
 
 
+void reg_set_custom_hdrs(struct reg *reg, const struct list *hdrs)
+{
+	struct le *le;
+
+	if (!reg)
+		return;
+
+	list_flush(&reg->custom_hdrs);
+
+	LIST_FOREACH(hdrs, le) {
+		struct sip_hdr *hdr = le->data;
+		char *buf = NULL;
+
+		if (re_sdprintf(&buf, "%r", &hdr->name))
+			return;
+
+		if (custom_hdrs_add(&reg->custom_hdrs, buf,
+				    "%r", &hdr->val)) {
+			mem_deref(buf);
+			return;
+		}
+
+		mem_deref(buf);
+	}
+}
+
+
 int reg_register(struct reg *reg, const char *reg_uri, const char *params,
 		 uint32_t regint, const char *outbound)
 {
@@ -209,17 +240,19 @@ int reg_register(struct reg *reg, const char *reg_uri, const char *params,
 
 	failed = sipreg_failed(reg->sipreg);
 	reg->sipreg = mem_deref(reg->sipreg);
-	err = sipreg_register(&reg->sipreg, uag_sip(), reg_uri,
+	err = sipreg_alloc(&reg->sipreg, uag_sip(), reg_uri,
 			      account_aor(acc),
 			      acc ? acc->dispname : NULL, account_aor(acc),
 			      regint, ua_local_cuser(reg->ua),
 			      routev[0] ? routev : NULL,
 			      routev[0] ? 1 : 0,
 			      reg->id,
-			      sip_auth_handler, ua_account(reg->ua), true,
+			      sip_auth_handler, acc, true,
 			      register_handler, reg,
 			      params[0] ? &params[1] : NULL,
-			      "Allow: %H\r\n", ua_print_allowed, reg->ua);
+			      "Allow: %H\r\n%H",
+			      ua_print_allowed, reg->ua,
+			      custom_hdrs_print, &reg->custom_hdrs);
 	if (err)
 		return err;
 
@@ -229,10 +262,18 @@ int reg_register(struct reg *reg, const char *reg_uri, const char *params,
 	if (acc && acc->fbregint)
 		err = sipreg_set_fbregint(reg->sipreg, acc->fbregint);
 
+	if (acc && acc->tcpsrcport)
+		sipreg_set_srcport(reg->sipreg, acc->tcpsrcport);
+
 	if (failed)
 		sipreg_incfailc(reg->sipreg);
 
-	return err;
+	if (err) {
+		reg->sipreg = mem_deref(reg->sipreg);
+		return err;
+	}
+
+	return sipreg_send(reg->sipreg);
 }
 
 
@@ -241,10 +282,17 @@ void reg_unregister(struct reg *reg)
 	if (!reg)
 		return;
 
-	reg->scode = 0;
-	reg->af    = 0;
+	sipreg_unregister(reg->sipreg);
+}
+
+
+void reg_stop(struct reg *reg)
+{
+	if (!reg)
+		return;
 
 	reg->sipreg = mem_deref(reg->sipreg);
+	reg->scode = 0;
 }
 
 
@@ -253,7 +301,7 @@ bool reg_isok(const struct reg *reg)
 	if (!reg || !reg->sipreg)
 		return false;
 
-	return sipreg_registered(reg->sipreg);
+	return sipreg_registered(reg->sipreg) && reg->scode==200;
 }
 
 
@@ -344,7 +392,7 @@ int reg_status(struct re_printf *pf, const struct reg *reg)
 		return 0;
 
 	pexpires = sipreg_proxy_expires(reg->sipreg);
-	fb = !pexpires && reg->scode;
+	fb = !pexpires && reg->scode && account_fbregint(ua_account(reg->ua));
 
 	if (pexpires) {
 		return re_hprintf(pf, " %s %s Expires %us",
@@ -365,3 +413,12 @@ int reg_af(const struct reg *reg)
 
 	return reg->af;
 }
+
+
+const struct sa *reg_laddr(const struct reg *reg)
+{
+	if (!reg)
+		return NULL;
+
+	return sipreg_laddr(reg->sipreg);
+}
