@@ -68,10 +68,84 @@ int test_account(void)
 	ASSERT_STREQ("bob@bob.com", account_stun_user(acc));
 	ASSERT_STREQ("taj:aa", account_stun_pass(acc));
 	ASSERT_STREQ("stunserver.org", account_stun_host(acc));
-	ASSERT_STREQ("no", account_mwi(acc));
-	ASSERT_STREQ("no", account_call_transfer(acc));
+	ASSERT_TRUE(!account_mwi(acc));
+	ASSERT_TRUE(!account_call_transfer(acc));
 
  out:
 	mem_deref(acc);
 	return err;
 }
+
+
+int test_account_uri_complete(void)
+{
+	static const struct test {
+		const char *in;
+		const char *out;
+	} testv[] = {
+
+		{ "192.168.1.2",
+		  "sip:192.168.1.2" },
+
+		{ "192.168.1.2:5677",
+		  "sip:192.168.1.2:5677", },
+
+		{ "user",
+		  "sip:user@proxy.com" },
+
+		{ "user@domain.com",
+		  "sip:user@domain.com" },
+
+		{ "user@domain.com:5677",
+		  "sip:user@domain.com:5677" },
+
+		{ "sip:office.local",
+		  "sip:office.local" },
+
+		{ "sip:user@domain.com",
+		  "sip:user@domain.com" },
+
+		{"[2113:1470:1f1b:24b::2]",
+		 "sip:[2113:1470:1f1b:24b::2]"},
+
+		{"[fe80::b62e:99ff:feee:268f]",
+		 "sip:[fe80::b62e:99ff:feee:268f]"},
+
+		{"x@[2113:1470:1f1b:24b::2]",
+		 "sip:x@[2113:1470:1f1b:24b::2]"},
+
+		{"[2113:1470:1f1b:24b::2]:5677",
+		 "sip:[2113:1470:1f1b:24b::2]:5677"},
+
+		{"x@[2113:1470:1f1b:24b::2]:5677",
+		 "sip:x@[2113:1470:1f1b:24b::2]:5677"},
+	};
+
+	struct mbuf *mb = NULL;
+	struct account *acc = NULL;
+	int err = 0;
+
+	err = account_alloc(&acc, "\"A\" <sip:A@proxy.com>");
+	TEST_ERR(err);
+
+	mb = mbuf_alloc(256);
+	ASSERT_TRUE(mb != NULL);
+
+	for (size_t i=0; i<RE_ARRAY_SIZE(testv); i++) {
+
+		const struct test *test = &testv[i];
+
+		err = account_uri_complete(acc, mb, test->in);
+		TEST_ERR_TXT(err, test->in);
+
+		TEST_STRCMP(test->out, str_len(test->out), mb->buf, mb->end);
+
+		mbuf_rewind(mb);
+	}
+
+ out:
+	mem_deref(mb);
+	mem_deref(acc);
+
+	return err;
+}
