@@ -15,10 +15,6 @@
 #    Sebastian Reimers
 #
 #
-# TODO:
-# - optimize regex functions
-# - count max y lines
-#
 
 import sys, os, re, fnmatch, getopt
 
@@ -39,27 +35,29 @@ class ccheck:
         self.cur_lineno = 0
         self.empty_lines_count = 0
         self.cc_count = 0
+        self.listfor_depth = 0
+        self.listfor_next = False
         self.files = {}
         self.extensions = ['c', 'cpp', 'h', 'mk', 'm4', 'py', 'm', 's', 'java',
                            'php']
 
         self.operators = ["do", "if", "for", "while", "switch"]
-        self.re_tab  = re.compile('(\w+\W*)\(')
-        self.re_else = re.compile('\s*\}\s*else')
-        self.re_inc  = re.compile('(^\s+\w+[+-]{2};)')
-        self.re_hex  = re.compile('0x([0-9A-Fa-f]+)')
+        self.re_tab  = re.compile(r'(\w+\W*)\(')
+        self.re_else = re.compile(r'\s*\}\s*else')
+        self.re_inc  = re.compile(r'(^\s+\w+[+-]{2};)')
+        self.re_hex  = re.compile(r'0x([0-9A-Fa-f]+)')
 
         # empty dict
         for e in self.extensions:
             self.files[e] = []
 
-        # todo: global config
         self.common_checks = [self.check_whitespace, self.check_termination,
                               self.check_hex_lowercase, self.check_pre_incr,
                               self.check_file_unix]
         self.funcmap = {
             'c':    [self.check_brackets, self.check_c_preprocessor,
-                     self.check_indent_tab],
+                     self.check_indent_tab, self.check_c11_err_handling,
+                     self.check_list_unlink, self.check_pri64],
             'h':    [self.check_brackets, self.check_indent_tab],
             'cpp':  [self.check_brackets, self.check_indent_tab],
             'mk':   [self.check_indent_tab],
@@ -179,7 +177,7 @@ class ccheck:
         if line[-2:] == ';;':
             self.error("has double semicolon")
 
-        if line[-2:] == ' ;' and re.search('[\S]+[ \t]+;$', line):
+        if line[-2:] == ' ;' and re.search(r'[\S]+[ \t]+;$', line):
             self.error("has spaces before terminator")
 
 
@@ -194,6 +192,65 @@ class ccheck:
                 self.error("C++ comment, use C comments /* ... */ instead")
 
 
+    #
+    # check wrong re list unlink/move handling
+    #
+    def check_list_unlink(self, line, len):
+        if self.cur_lineno == 1:
+            self.listfor_next = False
+            self.listfor_depth = 0
+
+        if (self.listfor_depth >= 1):
+            if '{' in line:
+                self.listfor_depth += 1
+                return
+            if '}' in line:
+                self.listfor_depth -= 1
+                if (self.listfor_depth == 0):
+                    self.listfor_next = False
+                return
+            if '->next' in line:
+                self.listfor_next = True
+                return
+            if not self.listfor_next and 'list_unlink' in line:
+                self.error("Use list_unlink() only after le->next "
+                           "(and use a while loop)")
+                self.listfor_next = False
+                self.listfor_depth = 0
+                return
+            if not self.listfor_next and 'list_move' in line:
+                self.error("Use list_move() only after le->next "
+                           "(and use a while loop)")
+                self.listfor_next = False
+                self.listfor_depth = 0
+                return
+            return
+
+        if 'LIST_FOREACH' in line:
+            self.listfor_depth = 1
+            return
+
+        if re.search('for.*->next.*{', line):
+            self.listfor_depth = 1
+            return
+
+        if re.search('while.*le.*{', line):
+            self.listfor_depth = 1
+            return
+
+
+    #
+    # check for wrong C11 return checks
+    #
+    def check_c11_err_handling(self, line, len):
+
+        if re.search('err.*=.*(mtx_|thrd_|cnd_|tss_)', line):
+            if re.search('(success|busy|error|nomem|timedout|alloc)', line):
+                return
+            self.error("Wrong C11 return check,"
+                    " use e.g. 'err = mtx_init(m,t) != thrd_success;'")
+
+
     #
     # check that C comments are not used
     #
@@ -213,7 +270,7 @@ class ccheck:
             self.cc_count = 0
 
         if cc:
-            self.error("C comment, use Perl-style comments # ... instead");
+            self.error("C comment, use Perl-style comments # ... instead")
 
 
     #
@@ -226,11 +283,7 @@ class ccheck:
 
         if l > max_x:
             self.error("line is too wide (" + str(l) + " - max " \
-                       + str(max_x) + ")");
-
-        #    TODO:
-        #    if ($line > $max_y) {
-        #      self.error("is too big ($lines lines - max $max_y)\n");
+                       + str(max_x) + ")")
 
 
     #
@@ -248,8 +301,6 @@ class ccheck:
     #
     # check for correct brackets usage in C/C++
     #
-    # TODO: this is too slow, optimize
-    #
     def check_brackets(self, line, len):
 
         m = self.re_tab.search(line)
@@ -276,7 +327,7 @@ class ccheck:
             return
 
         if line[-1] == '\r':
-            self.error("not in Unix format");
+            self.error("not in Unix format")
 
 
     #
@@ -288,9 +339,16 @@ class ccheck:
         if m:
             op = m.group(1)
             if op.find('++') != -1:
-                self.error("Use pre-increment: %s" % op);
+                self.error("Use pre-increment: %s" % op)
             else:
-                self.error("Use pre-decrement: %s" % op);
+                self.error("Use pre-decrement: %s" % op)
+
+    #
+    # check PRI Macros (not needed)
+    #
+    def check_pri64(self, line, len):
+        if re.search('_printf.*PRI.?64', line):
+            self.error("Use %Lu, %Lx or %Li instead of PRI*64 for [u]int64_t")
 
 
     def process_line(self, line, funcs, ext):
@@ -305,7 +363,7 @@ class ccheck:
             func(line, line_len)
 
         if ext in self.maxsize:
-            (x, y) = self.maxsize[ext];
+            (x, y) = self.maxsize[ext]
             self.check_xy_max(line, line_len, x)
 
 
@@ -317,11 +375,11 @@ class ccheck:
 
         self.cur_filename = filename
 
+        self.cur_lineno = 0
         while 1:
             lines = f.readlines(100000)
             if not lines:
                 break
-            self.cur_lineno = 0
             for line in lines:
                 self.cur_lineno += 1
                 self.process_line(line, funcs, ext)
