@@ -3,11 +3,14 @@
  *
  * Copyright (C) 2010 Alfred E. Heggestad
  */
+#ifdef HAVE_GETOPT
 #include <getopt.h>
+#endif
 #include <re.h>
 #include <baresip.h>
 #include "test.h"
 
+enum { ASYNC_WORKERS = 4 };
 
 typedef int (test_exec_h)(void);
 
@@ -20,7 +23,7 @@ struct test {
 
 static const struct test tests[] = {
 	TEST(test_account),
-	TEST(test_aulevel),
+	TEST(test_account_uri_complete),
 	TEST(test_call_answer),
 	TEST(test_call_answer_hangup_a),
 	TEST(test_call_answer_hangup_b),
@@ -40,37 +43,54 @@ static const struct test tests[] = {
 	TEST(test_call_tcp),
 	TEST(test_call_deny_udp),
 	TEST(test_call_transfer),
+	TEST(test_call_transfer_fail),
+	TEST(test_call_attended_transfer),
 	TEST(test_call_video),
+	TEST(test_call_change_videodir),
 	TEST(test_call_webrtc),
+	TEST(test_call_bundle),
+	TEST(test_call_ipv6ll),
+	TEST(test_call_100rel_audio),
+	TEST(test_call_100rel_video),
+	TEST(test_call_hold_resume),
 	TEST(test_cmd),
 	TEST(test_cmd_long),
 	TEST(test_contact),
 	TEST(test_event),
-	TEST(test_h264),
+	TEST(test_jbuf),
+	TEST(test_jbuf_adaptive),
+	TEST(test_jbuf_adaptive_video),
 	TEST(test_message),
 	TEST(test_network),
 	TEST(test_play),
+	TEST(test_stunuri),
 	TEST(test_ua_alloc),
 	TEST(test_ua_options),
+	TEST(test_ua_refer),
 	TEST(test_ua_register),
 	TEST(test_ua_register_auth),
 	TEST(test_ua_register_auth_dns),
 	TEST(test_ua_register_dns),
 	TEST(test_uag_find_param),
 	TEST(test_video),
+	TEST(test_clean_number),
+	TEST(test_clean_number_only_numeric),
 };
 
 
 static int run_one_test(const struct test *test)
 {
+	struct config *config = conf_config();
+	enum rtp_receive_mode rxmode = config->avt.rxmode;
 	int err;
 
-	re_printf("[ RUN      ] %s\n", test->name);
+	re_printf("[ RUN      ] %s (rx %s)\n",
+		  test->name, rtp_receive_mode_str(rxmode));
 
 	err = test->exec();
 	if (err) {
-		warning("%s: test failed (%m)\n",
-			test->name, err);
+		warning("%s (rx %s): test failed (%m)\n",
+			test->name, rtp_receive_mode_str(rxmode), err);
 		return err;
 	}
 
@@ -80,19 +100,50 @@ static int run_one_test(const struct test *test)
 }
 
 
+static int run_one_test_rxmode(const struct test *test, struct pl *rxmode)
+{
+	struct config *config = conf_config();
+	int err;
+
+	if (pl_isset(rxmode)) {
+		config->avt.rxmode = resolve_receive_mode(rxmode);
+		err = run_one_test(test);
+		if (err)
+			return err;
+	}
+	else {
+		config->avt.rxmode = RECEIVE_MODE_MAIN;
+		err = run_one_test(test);
+		if (err)
+			return err;
+
+		config->avt.rxmode = RECEIVE_MODE_THREAD;
+		err = run_one_test(test);
+		if (err)
+			return err;
+	}
+
+	return 0;
+}
+
+
 static int run_tests(void)
 {
 	size_t i;
+	struct config *config = conf_config();
+	enum rtp_receive_mode rxmode = config->avt.rxmode;
 	int err;
 
-	for (i=0; i<ARRAY_SIZE(tests); i++) {
+	for (i=0; i<RE_ARRAY_SIZE(tests); i++) {
 
-		re_printf("[ RUN      ] %s\n", tests[i].name);
+		re_printf("[ RUN      ] %s (rx %s)\n",
+			  tests[i].name, rtp_receive_mode_str(rxmode));
 
 		err = tests[i].exec();
 		if (err) {
-			warning("%s: test failed (%m)\n",
-				tests[i].name, err);
+			warning("%s (rx %s): test failed (%m)\n",
+				tests[i].name, rtp_receive_mode_str(rxmode),
+				err);
 			return err;
 		}
 
@@ -103,11 +154,38 @@ static int run_tests(void)
 }
 
 
+static int run_tests_rxmode(struct pl *rxmode)
+{
+	struct config *config = conf_config();
+	int err;
+
+	if (pl_isset(rxmode)) {
+		config->avt.rxmode = resolve_receive_mode(rxmode);
+		err = run_tests();
+		if (err)
+			return err;
+	}
+	else {
+		config->avt.rxmode = RECEIVE_MODE_MAIN;
+		err = run_tests();
+		if (err)
+			return err;
+
+		config->avt.rxmode = RECEIVE_MODE_THREAD;
+		err = run_tests();
+		if (err)
+			return err;
+	}
+
+	return 0;
+}
+
+
 static void test_listcases(void)
 {
 	size_t i, n;
 
-	n = ARRAY_SIZE(tests);
+	n = RE_ARRAY_SIZE(tests);
 
 	(void)re_printf("\n%zu test cases:\n", n);
 
@@ -126,7 +204,7 @@ static const struct test *find_test(const char *name)
 {
 	size_t i;
 
-	for (i=0; i<ARRAY_SIZE(tests); i++) {
+	for (i=0; i<RE_ARRAY_SIZE(tests); i++) {
 
 		if (0 == str_casecmp(name, tests[i].name))
 			return &tests[i];
@@ -151,26 +229,39 @@ static void usage(void)
 			 "Usage: selftest [options] <testcases..>\n"
 			 "options:\n"
 			 "\t-l               List all testcases and exit\n"
+			 "\t-r <rxmode>      RTP RX processing mode "
+			 "[main, thread]\n"
 			 "\t-v               Verbose output (INFO level)\n"
 			 );
 }
 
 
+static const char *modconfig =
+	"ausrc_format    s16\n";
+
+
 int main(int argc, char *argv[])
 {
+	struct memstat mstat;
 	struct config *config;
-	size_t i, ntests;
+	size_t ntests;
+	struct sa sa;
 	bool verbose = false;
+	struct pl rxmode = PL_INIT;
 	int err;
 
+	libre_exception_btrace(true);
+
 	err = libre_init();
 	if (err)
 		return err;
 
 	log_enable_info(false);
+	re_thread_async_init(ASYNC_WORKERS);
 
+#ifdef HAVE_GETOPT
 	for (;;) {
-		const int c = getopt(argc, argv, "hlv");
+		const int c = getopt(argc, argv, "hlvr:");
 		if (0 > c)
 			break;
 
@@ -185,6 +276,10 @@ int main(int argc, char *argv[])
 			test_listcases();
 			return 0;
 
+		case 'r':
+			pl_set_str(&rxmode, optarg);
+			break;
+
 		case 'v':
 			if (verbose)
 				log_enable_debug(true);
@@ -201,11 +296,22 @@ int main(int argc, char *argv[])
 	if (argc >= (optind + 1))
 		ntests = argc - optind;
 	else
-		ntests = ARRAY_SIZE(tests);
+		ntests = RE_ARRAY_SIZE(tests);
+#else
+	(void)argc;
+	(void)argv;
+	ntests = RE_ARRAY_SIZE(tests);
+#endif
 
 	re_printf("running baresip selftest version %s with %zu tests\n",
 		  BARESIP_VERSION, ntests);
 
+	err = conf_configure_buf((uint8_t *)modconfig, str_len(modconfig));
+	if (err) {
+		warning("main: configure failed: %m\n", err);
+		goto out;
+	}
+
 	/* note: run SIP-traffic on localhost */
 	config = conf_config();
 	if (!config) {
@@ -213,26 +319,27 @@ int main(int argc, char *argv[])
 		goto out;
 	}
 
-	str_ncpy(config->net.ifname, "127.0.0.1", sizeof(config->net.ifname));
-
 	err = baresip_init(config);
+	err = sa_set_str(&sa, "127.0.0.1", 0);
+	err |= net_add_address(baresip_network(), &sa);
 	if (err)
 		goto out;
 
-	str_ncpy(config->sip.local, "127.0.0.1:0", sizeof(config->sip.local));
+	str_ncpy(config->sip.local, "0.0.0.0:0", sizeof(config->sip.local));
 	config->sip.verify_server = false;
 
 	uag_set_exit_handler(ua_exit_handler, NULL);
 
+#ifdef HAVE_GETOPT
 	if (argc >= (optind + 1)) {
 
-		for (i=0; i<ntests; i++) {
+		for (size_t i=0; i<ntests; i++) {
 			const char *name = argv[optind + i];
 			const struct test *test;
 
 			test = find_test(name);
 			if (test) {
-				err = run_one_test(test);
+				err = run_one_test_rxmode(test, &rxmode);
 				if (err)
 					goto out;
 			}
@@ -246,10 +353,15 @@ int main(int argc, char *argv[])
 		}
 	}
 	else {
-		err = run_tests();
+		err = run_tests_rxmode(&rxmode);
 		if (err)
 			goto out;
 	}
+#else
+	err = run_tests_rxmode(&rxmode);
+	if (err)
+		goto out;
+#endif
 
 #if 1
 	ua_stop_all(true);
@@ -261,17 +373,26 @@ int main(int argc, char *argv[])
  out:
 	if (err) {
 		warning("test failed (%m)\n", err);
-		re_printf("%H\n", re_debug, 0);
+		re_printf("%H\n", re_debug, NULL);
 	}
 	ua_stop_all(true);
 	ua_close();
+	conf_close();
 
 	baresip_close();
 
-	libre_close();
+	re_thread_async_close();
 
 	tmr_debug();
+
+	libre_close();
+
 	mem_debug();
 
+	if (0 == mem_get_stat(&mstat)) {
+		if (mstat.bytes_cur || mstat.blocks_cur)
+			return 2;
+	}
+
 	return err;
 }
