@@ -23,7 +23,7 @@ struct hdr {
 struct videnc_state {
 	double fps;
 	videnc_packet_h *pkth;
-	void *arg;
+	const struct video *vid;
 };
 
 struct viddec_state {
@@ -55,7 +55,7 @@ static void decode_destructor(void *arg)
 static int mock_encode_update(struct videnc_state **vesp,
 			      const struct vidcodec *vc,
 			      struct videnc_param *prm, const char *fmtp,
-			      videnc_packet_h *pkth, void *arg)
+			      videnc_packet_h *pkth, const struct video *vid)
 {
 	struct videnc_state *ves;
 	(void)fmtp;
@@ -76,7 +76,7 @@ static int mock_encode_update(struct videnc_state **vesp,
 
 	ves->fps     = prm->fps;
 	ves->pkth    = pkth;
-	ves->arg     = arg;
+	ves->vid     = vid;
 
 	return 0;
 }
@@ -105,7 +105,7 @@ static int mock_encode(struct videnc_state *ves, bool update,
 	rtp_ts = video_calc_rtp_timestamp_fix(timestamp);
 
 	err = ves->pkth(true, rtp_ts, hdr->buf, hdr->end,
-			payload, sizeof(payload), ves->arg);
+			payload, sizeof(payload), ves->vid);
 	if (err)
 		goto out;
 
@@ -117,11 +117,13 @@ static int mock_encode(struct videnc_state *ves, bool update,
 
 
 static int mock_decode_update(struct viddec_state **vdsp,
-			      const struct vidcodec *vc, const char *fmtp)
+			      const struct vidcodec *vc, const char *fmtp,
+			      const struct video *vid)
 {
 	struct viddec_state *vds;
 	(void)vc;
 	(void)fmtp;
+	(void)vid;
 
 	if (!vdsp)
 		return EINVAL;
@@ -142,18 +144,17 @@ static int mock_decode_update(struct viddec_state **vdsp,
 
 
 static int mock_decode(struct viddec_state *vds, struct vidframe *frame,
-		       bool *intra, bool marker, uint16_t seq, struct mbuf *mb)
+		       struct viddec_packet *pkt)
 {
 	struct vidsz size;
 	struct hdr hdr;
 	int err, i;
-	(void)marker;
-	(void)seq;
 
-	if (!vds || !frame || !intra || !mb)
+	if (!vds || !frame || !pkt || !pkt->mb)
 		return EINVAL;
 
-	*intra = false;
+	pkt->intra = false;
+	struct mbuf *mb = pkt->mb;
 
 	err = hdr_decode(&hdr, mb);
 	if (err) {
